#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that represents a Telegram Venue."""

from typing import TYPE_CHECKING

from telegram._files.location import Location
from telegram._telegramobject import TelegramObject
from telegram._utils.argumentparsing import de_json_optional
from telegram._utils.types import JSONDict

if TYPE_CHECKING:
    from telegram import Bot


class Venue(TelegramObject):
    """This object represents a venue.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`location` and :attr:`title` are equal.

    Note:
      Foursquare details and Google Place details are mutually exclusive. However, this
      behaviour is undocumented and might be changed by Telegram.

    Args:
        location (:class:`telegram.Location`): Venue location.
        title (:obj:`str`): Name of the venue.
        address (:obj:`str`): Address of the venue.
        foursquare_id (:obj:`str`, optional): Foursquare identifier of the venue.
        foursquare_type (:obj:`str`, optional): Foursquare type of the venue. (For example,
            "arts_entertainment/default", "arts_entertainment/aquarium" or "food/icecream".)
        google_place_id (:obj:`str`, optional): Google Places identifier of the venue.
        google_place_type (:obj:`str`, optional): Google Places type of the venue. (See
            `supported types <https://developers.google.com/maps/documentation/places/web-service\
            /place-types>`_.)

    Attributes:
        location (:class:`telegram.Location`): Venue location.
        title (:obj:`str`): Name of the venue.
        address (:obj:`str`): Address of the venue.
        foursquare_id (:obj:`str`): Optional. Foursquare identifier of the venue.
        foursquare_type (:obj:`str`): Optional. Foursquare type of the venue. (For example,
            "arts_entertainment/default", "arts_entertainment/aquarium" or "food/icecream".)
        google_place_id (:obj:`str`): Optional. Google Places identifier of the venue.
        google_place_type (:obj:`str`): Optional. Google Places type of the venue. (See
            `supported types <https://developers.google.com/maps/documentation/places/web-service\
            /place-types>`_.)

    """

    __slots__ = (
        "address",
        "foursquare_id",
        "foursquare_type",
        "google_place_id",
        "google_place_type",
        "location",
        "title",
    )

    def __init__(
        self,
        location: Location,
        title: str,
        address: str,
        foursquare_id: str | None = None,
        foursquare_type: str | None = None,
        google_place_id: str | None = None,
        google_place_type: str | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)

        # Required
        self.location: Location = location
        self.title: str = title
        self.address: str = address
        # Optionals
        self.foursquare_id: str | None = foursquare_id
        self.foursquare_type: str | None = foursquare_type
        self.google_place_id: str | None = google_place_id
        self.google_place_type: str | None = google_place_type

        self._id_attrs = (self.location, self.title)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "Venue":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["location"] = de_json_optional(data.get("location"), Location, bot)

        return super().de_json(data=data, bot=bot)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that represents a Telegram ReplyKeyboardMarkup."""

from collections.abc import Sequence
from typing import Final

from telegram import constants
from telegram._keyboardbutton import KeyboardButton
from telegram._telegramobject import TelegramObject
from telegram._utils.markup import check_keyboard_type
from telegram._utils.types import JSONDict


class ReplyKeyboardMarkup(TelegramObject):
    """This object represents a custom keyboard with reply options. Not supported in channels and
    for messages sent on behalf of a Telegram Business account.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their size of :attr:`keyboard` and all the buttons are equal.

    .. figure:: https://core.telegram.org/file/464001950/1191a/2RwpmgU-swU.123554/b5\
        0478c124d5914c23
        :align: center

        A reply keyboard with reply options.

    .. seealso::
        Another kind of keyboard would be the :class:`telegram.InlineKeyboardMarkup`.

    Examples:
        * Example usage: A user requests to change the bot's language, bot replies to the request
          with a keyboard to select the new language. Other users in the group don't see
          the keyboard.
        * :any:`Conversation Bot <examples.conversationbot>`
        * :any:`Conversation Bot 2 <examples.conversationbot2>`

    Args:
        keyboard (Sequence[Sequence[:obj:`str` | :class:`telegram.KeyboardButton`]]): Array of
            button rows, each represented by an Array of :class:`telegram.KeyboardButton` objects.
        resize_keyboard (:obj:`bool`, optional): Requests clients to resize the keyboard vertically
            for optimal fit (e.g., make the keyboard smaller if there are just two rows of
            buttons). Defaults to :obj:`False`, in which case the custom keyboard is always of the
            same height as the app's standard keyboard.
        one_time_keyboard (:obj:`bool`, optional): Requests clients to hide the keyboard as soon as
            it's been used. The keyboard will still be available, but clients will automatically
            display the usual letter-keyboard in the chat - the user can press a special button in
            the input field to see the custom keyboard again. Defaults to :obj:`False`.
        selective (:obj:`bool`, optional): Use this parameter if you want to show the keyboard to
            specific users only. Targets:

            1) Users that are @mentioned in the :attr:`~telegram.Message.text` of the
               :class:`telegram.Message` object.
            2) If the bot's message is a reply to a message in the same chat and forum topic,
                sender of the original message.

            Defaults to :obj:`False`.

        input_field_placeholder (:obj:`str`, optional): The placeholder to be shown in the input
            field when the keyboard is active;
            :tg-const:`telegram.ReplyKeyboardMarkup.MIN_INPUT_FIELD_PLACEHOLDER`-
            :tg-const:`telegram.ReplyKeyboardMarkup.MAX_INPUT_FIELD_PLACEHOLDER`
            characters.

            .. versionadded:: 13.7
        is_persistent (:obj:`bool`, optional): Requests clients to always show the keyboard when
            the regular keyboard is hidden. Defaults to :obj:`False`, in which case the custom
            keyboard can be hidden and opened with a keyboard icon.

            .. versionadded:: 20.0

    Attributes:
        keyboard (tuple[tuple[:class:`telegram.KeyboardButton`]]): Array of button rows,
            each represented by an Array of :class:`telegram.KeyboardButton` objects.
        resize_keyboard (:obj:`bool`): Optional. Requests clients to resize the keyboard vertically
            for optimal fit (e.g., make the keyboard smaller if there are just two rows of
            buttons). Defaults to :obj:`False`, in which case the custom keyboard is always of the
            same height as the app's standard keyboard.
        one_time_keyboard (:obj:`bool`): Optional. Requests clients to hide the keyboard as soon as
            it's been used. The keyboard will still be available, but clients will automatically
            display the usual letter-keyboard in the chat - the user can press a special button in
            the input field to see the custom keyboard again. Defaults to :obj:`False`.
        selective (:obj:`bool`): Optional. Show the keyboard to specific users only.
            Targets:

            1) Users that are @mentioned in the :attr:`~telegram.Message.text` of the
               :class:`telegram.Message` object.
            2) If the bot's message is a reply to a message in the same chat and forum topic,
                sender of the original message.

            Defaults to :obj:`False`.

        input_field_placeholder (:obj:`str`): Optional. The placeholder to be shown in the input
            field when the keyboard is active;
            :tg-const:`telegram.ReplyKeyboardMarkup.MIN_INPUT_FIELD_PLACEHOLDER`-
            :tg-const:`telegram.ReplyKeyboardMarkup.MAX_INPUT_FIELD_PLACEHOLDER`
            characters.

            .. versionadded:: 13.7
        is_persistent (:obj:`bool`): Optional. Requests clients to always show the keyboard when
            the regular keyboard is hidden. If :obj:`False`, the custom keyboard can be hidden and
            opened with a keyboard icon.

            .. versionadded:: 20.0

    """

    __slots__ = (
        "input_field_placeholder",
        "is_persistent",
        "keyboard",
        "one_time_keyboard",
        "resize_keyboard",
        "selective",
    )

    def __init__(
        self,
        keyboard: Sequence[Sequence[str | KeyboardButton]],
        resize_keyboard: bool | None = None,
        one_time_keyboard: bool | None = None,
        selective: bool | None = None,
        input_field_placeholder: str | None = None,
        is_persistent: bool | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        if not check_keyboard_type(keyboard):
            raise ValueError(
                "The parameter `keyboard` should be a sequence of sequences of "
                "strings or KeyboardButtons"
            )

        # Required
        self.keyboard: tuple[tuple[KeyboardButton, ...], ...] = tuple(
            tuple(KeyboardButton(button) if isinstance(button, str) else button for button in row)
            for row in keyboard
        )

        # Optionals
        self.resize_keyboard: bool | None = resize_keyboard
        self.one_time_keyboard: bool | None = one_time_keyboard
        self.selective: bool | None = selective
        self.input_field_placeholder: str | None = input_field_placeholder
        self.is_persistent: bool | None = is_persistent

        self._id_attrs = (self.keyboard,)

        self._freeze()

    @classmethod
    def from_button(
        cls,
        button: KeyboardButton | str,
        resize_keyboard: bool = False,
        one_time_keyboard: bool = False,
        selective: bool = False,
        input_field_placeholder: str | None = None,
        is_persistent: bool | None = None,
        **kwargs: object,
    ) -> "ReplyKeyboardMarkup":
        """Shortcut for::

            ReplyKeyboardMarkup([[button]], **kwargs)

        Return a ReplyKeyboardMarkup from a single KeyboardButton.

        Args:
            button (:class:`telegram.KeyboardButton` | :obj:`str`): The button to use in
                the markup.
            resize_keyboard (:obj:`bool`, optional): Requests clients to resize the keyboard
                vertically for optimal fit (e.g., make the keyboard smaller if there are just two
                rows of buttons). Defaults to :obj:`False`, in which case the custom keyboard is
                always of the same height as the app's standard keyboard.
            one_time_keyboard (:obj:`bool`, optional): Requests clients to hide the keyboard as
                soon as it's been used. The keyboard will still be available, but clients will
                automatically display the usual letter-keyboard in the chat - the user can press
                a special button in the input field to see the custom keyboard again.
                Defaults to :obj:`False`.
            selective (:obj:`bool`, optional): Use this parameter if you want to show the keyboard
                to specific users only. Targets:

                1) Users that are @mentioned in the text of the Message object.
                2) If the bot's message is a reply to a message in the same chat and forum topic,
                    sender of the original message.

                Defaults to :obj:`False`.

            input_field_placeholder (:obj:`str`): Optional. The placeholder shown in the input
                field when the reply is active.

                .. versionadded:: 13.7
            is_persistent (:obj:`bool`): Optional. Requests clients to always show the keyboard
                when the regular keyboard is hidden. Defaults to :obj:`False`, in which case the
                custom keyboard can be hidden and opened with a keyboard icon.

                .. versionadded:: 20.0
        """
        return cls(
            [[button]],
            resize_keyboard=resize_keyboard,
            one_time_keyboard=one_time_keyboard,
            selective=selective,
            input_field_placeholder=input_field_placeholder,
            is_persistent=is_persistent,
            **kwargs,  # type: ignore[arg-type]
        )

    @classmethod
    def from_row(
        cls,
        button_row: Sequence[str | KeyboardButton],
        resize_keyboard: bool = False,
        one_time_keyboard: bool = False,
        selective: bool = False,
        input_field_placeholder: str | None = None,
        is_persistent: bool | None = None,
        **kwargs: object,
    ) -> "ReplyKeyboardMarkup":
        """Shortcut for::

            ReplyKeyboardMarkup([button_row], **kwargs)

        Return a ReplyKeyboardMarkup from a single row of KeyboardButtons.

        Args:
            button_row (Sequence[:class:`telegram.KeyboardButton` | :obj:`str`]): The button to
                use in the markup.

                .. versionchanged:: 20.0
                    |sequenceargs|
            resize_keyboard (:obj:`bool`, optional): Requests clients to resize the keyboard
                vertically for optimal fit (e.g., make the keyboard smaller if there are just two
                rows of buttons). Defaults to :obj:`False`, in which case the custom keyboard is
                always of the same height as the app's standard keyboard.
            one_time_keyboard (:obj:`bool`, optional): Requests clients to hide the keyboard as
                soon as it's been used. The keyboard will still be available, but clients will
                automatically display the usual letter-keyboard in the chat - the user can press
                a special button in the input field to see the custom keyboard again.
                Defaults to :obj:`False`.
            selective (:obj:`bool`, optional): Use this parameter if you want to show the keyboard
                to specific users only. Targets:

                1) Users that are @mentioned in the text of the Message object.
                2) If the bot's message is a reply to a message in the same chat and forum topic,
                    sender of the original message.

                Defaults to :obj:`False`.

            input_field_placeholder (:obj:`str`): Optional. The placeholder shown in the input
                field when the reply is active.

                .. versionadded:: 13.7
            is_persistent (:obj:`bool`): Optional. Requests clients to always show the keyboard
                when the regular keyboard is hidden. Defaults to :obj:`False`, in which case the
                custom keyboard can be hidden and opened with a keyboard icon.

                .. versionadded:: 20.0

        """
        return cls(
            [button_row],
            resize_keyboard=resize_keyboard,
            one_time_keyboard=one_time_keyboard,
            selective=selective,
            input_field_placeholder=input_field_placeholder,
            is_persistent=is_persistent,
            **kwargs,  # type: ignore[arg-type]
        )

    @classmethod
    def from_column(
        cls,
        button_column: Sequence[str | KeyboardButton],
        resize_keyboard: bool = False,
        one_time_keyboard: bool = False,
        selective: bool = False,
        input_field_placeholder: str | None = None,
        is_persistent: bool | None = None,
        **kwargs: object,
    ) -> "ReplyKeyboardMarkup":
        """Shortcut for::

            ReplyKeyboardMarkup([[button] for button in button_column], **kwargs)

        Return a ReplyKeyboardMarkup from a single column of KeyboardButtons.

        Args:
            button_column (Sequence[:class:`telegram.KeyboardButton` | :obj:`str`]): The button
                to use in the markup.

                .. versionchanged:: 20.0
                    |sequenceargs|
            resize_keyboard (:obj:`bool`, optional): Requests clients to resize the keyboard
                vertically for optimal fit (e.g., make the keyboard smaller if there are just two
                rows of buttons). Defaults to :obj:`False`, in which case the custom keyboard is
                always of the same height as the app's standard keyboard.
            one_time_keyboard (:obj:`bool`, optional): Requests clients to hide the keyboard as
                soon as it's been used. The keyboard will still be available, but clients will
                automatically display the usual letter-keyboard in the chat - the user can press
                a special button in the input field to see the custom keyboard again.
                Defaults to :obj:`False`.
            selective (:obj:`bool`, optional): Use this parameter if you want to show the keyboard
                to specific users only. Targets:

                1) Users that are @mentioned in the text of the Message object.
                2) If the bot's message is a reply to a message in the same chat and forum topic,
                    sender of the original message.

                Defaults to :obj:`False`.

            input_field_placeholder (:obj:`str`): Optional. The placeholder shown in the input
                field when the reply is active.

                .. versionadded:: 13.7
            is_persistent (:obj:`bool`): Optional. Requests clients to always show the keyboard
                when the regular keyboard is hidden. Defaults to :obj:`False`, in which case the
                custom keyboard can be hidden and opened with a keyboard icon.

                .. versionadded:: 20.0

        """
        button_grid = [[button] for button in button_column]
        return cls(
            button_grid,
            resize_keyboard=resize_keyboard,
            one_time_keyboard=one_time_keyboard,
            selective=selective,
            input_field_placeholder=input_field_placeholder,
            is_persistent=is_persistent,
            **kwargs,  # type: ignore[arg-type]
        )

    MIN_INPUT_FIELD_PLACEHOLDER: Final[int] = constants.ReplyLimit.MIN_INPUT_FIELD_PLACEHOLDER
    """:const:`telegram.constants.ReplyLimit.MIN_INPUT_FIELD_PLACEHOLDER`

    .. versionadded:: 20.0
    """
    MAX_INPUT_FIELD_PLACEHOLDER: Final[int] = constants.ReplyLimit.MAX_INPUT_FIELD_PLACEHOLDER
    """:const:`telegram.constants.ReplyLimit.MAX_INPUT_FIELD_PLACEHOLDER`

    .. versionadded:: 20.0
    """



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program. If not, see [http://www.gnu.org/licenses/].
import datetime as dtm

import pytest

from telegram import (
    StarTransaction,
    StarTransactions,
    TransactionPartnerOther,
    TransactionPartnerUser,
    User,
)
from telegram._utils.datetime import UTC, from_timestamp, to_timestamp
from tests.auxil.slots import mro_slots


def star_transaction_factory():
    return StarTransaction(
        id=StarTransactionTestBase.id,
        amount=StarTransactionTestBase.amount,
        nanostar_amount=StarTransactionTestBase.nanostar_amount,
        date=from_timestamp(StarTransactionTestBase.date),
        source=StarTransactionTestBase.source,
        receiver=StarTransactionTestBase.receiver,
    )


@pytest.fixture
def star_transaction():
    return star_transaction_factory()


@pytest.fixture
def star_transactions():
    return StarTransactions(
        transactions=[
            star_transaction_factory(),
            star_transaction_factory(),
        ]
    )


class StarTransactionTestBase:
    id = "2"
    amount = 2
    nanostar_amount = 365
    date = to_timestamp(dtm.datetime(2024, 1, 1, 0, 0, 0, 0, tzinfo=UTC))
    source = TransactionPartnerUser(
        transaction_type="premium_purchase",
        user=User(
            id=2,
            is_bot=False,
            first_name="first_name",
        ),
    )
    receiver = TransactionPartnerOther()


class TestStarTransactionWithoutRequest(StarTransactionTestBase):
    def test_slot_behaviour(self, star_transaction):
        inst = star_transaction
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "id": self.id,
            "amount": self.amount,
            "nanostar_amount": self.nanostar_amount,
            "date": self.date,
            "source": self.source.to_dict(),
            "receiver": self.receiver.to_dict(),
        }
        st = StarTransaction.de_json(json_dict, offline_bot)
        assert st.api_kwargs == {}
        assert st.id == self.id
        assert st.amount == self.amount
        assert st.nanostar_amount == self.nanostar_amount
        assert st.date == from_timestamp(self.date)
        assert st.source == self.source
        assert st.receiver == self.receiver

    def test_de_json_star_transaction_localization(
        self, tz_bot, offline_bot, raw_bot, star_transaction
    ):
        json_dict = star_transaction.to_dict()
        st_raw = StarTransaction.de_json(json_dict, raw_bot)
        st_bot = StarTransaction.de_json(json_dict, offline_bot)
        st_tz = StarTransaction.de_json(json_dict, tz_bot)

        # comparing utcoffsets because comparing timezones is unpredicatable
        st_offset = st_tz.date.utcoffset()
        tz_bot_offset = tz_bot.defaults.tzinfo.utcoffset(st_tz.date.replace(tzinfo=None))

        assert st_raw.date.tzinfo == UTC
        assert st_bot.date.tzinfo == UTC
        assert st_offset == tz_bot_offset

    def test_to_dict(self, star_transaction):
        expected_dict = {
            "id": self.id,
            "amount": self.amount,
            "nanostar_amount": self.nanostar_amount,
            "date": self.date,
            "source": self.source.to_dict(),
            "receiver": self.receiver.to_dict(),
        }
        assert star_transaction.to_dict() == expected_dict

    def test_equality(self):
        a = StarTransaction(
            id=self.id,
            amount=self.amount,
            date=self.date,
            source=self.source,
            receiver=self.receiver,
        )
        b = StarTransaction(
            id=self.id,
            amount=self.amount,
            date=None,
            source=self.source,
            receiver=self.receiver,
        )
        c = StarTransaction(
            id="3",
            amount=3,
            date=to_timestamp(dtm.datetime.utcnow()),
            source=TransactionPartnerUser(
                transaction_type="other_type",
                user=User(
                    id=3,
                    is_bot=False,
                    first_name="first_name",
                ),
            ),
            receiver=TransactionPartnerOther(),
        )

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)


class StarTransactionsTestBase:
    transactions = [star_transaction_factory(), star_transaction_factory()]


class TestStarTransactionsWithoutRequest(StarTransactionsTestBase):
    def test_slot_behaviour(self, star_transactions):
        inst = star_transactions
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "transactions": [t.to_dict() for t in self.transactions],
        }
        st = StarTransactions.de_json(json_dict, offline_bot)
        assert st.api_kwargs == {}
        assert st.transactions == tuple(self.transactions)

    def test_to_dict(self, star_transactions):
        expected_dict = {
            "transactions": [t.to_dict() for t in self.transactions],
        }
        assert star_transactions.to_dict() == expected_dict

    def test_equality(self, star_transaction):
        a = StarTransactions(
            transactions=self.transactions,
        )
        b = StarTransactions(
            transactions=self.transactions,
        )
        c = StarTransactions(
            transactions=[star_transaction],
        )

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains the classes that represent Telegram InlineQueryResultArticle."""

from typing import TYPE_CHECKING

from telegram._inline.inlinekeyboardmarkup import InlineKeyboardMarkup
from telegram._inline.inlinequeryresult import InlineQueryResult
from telegram._utils.types import JSONDict
from telegram.constants import InlineQueryResultType

if TYPE_CHECKING:
    from telegram import InputMessageContent


class InlineQueryResultArticle(InlineQueryResult):
    """This object represents a Telegram InlineQueryResultArticle.

    Examples:
        :any:`Inline Bot <examples.inlinebot>`

    .. versionchanged:: 20.5
      Removed the deprecated arguments and attributes ``thumb_*``.

    .. versionchanged:: 21.11
        Removed the deprecated argument and attribute ``hide_url``.

    Args:
        id (:obj:`str`): Unique identifier for this result,
            :tg-const:`telegram.InlineQueryResult.MIN_ID_LENGTH`-
            :tg-const:`telegram.InlineQueryResult.MAX_ID_LENGTH` Bytes.
        title (:obj:`str`): Title of the result.
        input_message_content (:class:`telegram.InputMessageContent`): Content of the message to
            be sent.
        reply_markup (:class:`telegram.InlineKeyboardMarkup`, optional): Inline keyboard attached
            to the message.
        url (:obj:`str`, optional): URL of the result.

            Tip:
                Pass an empty string as URL if you don't want the URL to be shown in the message.
        description (:obj:`str`, optional): Short description of the result.
        thumbnail_url (:obj:`str`, optional): Url of the thumbnail for the result.

            .. versionadded:: 20.2
        thumbnail_width (:obj:`int`, optional): Thumbnail width.

            .. versionadded:: 20.2
        thumbnail_height (:obj:`int`, optional): Thumbnail height.

            .. versionadded:: 20.2

    Attributes:
        type (:obj:`str`): :tg-const:`telegram.constants.InlineQueryResultType.ARTICLE`.
        id (:obj:`str`): Unique identifier for this result,
            :tg-const:`telegram.InlineQueryResult.MIN_ID_LENGTH`-
            :tg-const:`telegram.InlineQueryResult.MAX_ID_LENGTH` Bytes.
        title (:obj:`str`): Title of the result.
        input_message_content (:class:`telegram.InputMessageContent`): Content of the message to
            be sent.
        reply_markup (:class:`telegram.InlineKeyboardMarkup`): Optional. Inline keyboard attached
            to the message.
        url (:obj:`str`): Optional. URL of the result.
        description (:obj:`str`): Optional. Short description of the result.
        thumbnail_url (:obj:`str`): Optional. Url of the thumbnail for the result.

            .. versionadded:: 20.2
        thumbnail_width (:obj:`int`): Optional. Thumbnail width.

            .. versionadded:: 20.2
        thumbnail_height (:obj:`int`): Optional. Thumbnail height.

            .. versionadded:: 20.2

    """

    __slots__ = (
        "description",
        "input_message_content",
        "reply_markup",
        "thumbnail_height",
        "thumbnail_url",
        "thumbnail_width",
        "title",
        "url",
    )

    def __init__(
        self,
        id: str,  # pylint: disable=redefined-builtin
        title: str,
        input_message_content: "InputMessageContent",
        reply_markup: InlineKeyboardMarkup | None = None,
        url: str | None = None,
        description: str | None = None,
        thumbnail_url: str | None = None,
        thumbnail_width: int | None = None,
        thumbnail_height: int | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        # Required
        super().__init__(InlineQueryResultType.ARTICLE, id, api_kwargs=api_kwargs)
        with self._unfrozen():
            self.title: str = title
            self.input_message_content: InputMessageContent = input_message_content

            # Optional
            self.reply_markup: InlineKeyboardMarkup | None = reply_markup
            self.url: str | None = url
            self.description: str | None = description
            self.thumbnail_url: str | None = thumbnail_url
            self.thumbnail_width: int | None = thumbnail_width
            self.thumbnail_height: int | None = thumbnail_height



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that represents a Telegram ShippingQuery."""

from collections.abc import Sequence
from typing import TYPE_CHECKING

from telegram._payment.shippingaddress import ShippingAddress
from telegram._telegramobject import TelegramObject
from telegram._user import User
from telegram._utils.argumentparsing import de_json_optional
from telegram._utils.defaultvalue import DEFAULT_NONE
from telegram._utils.types import JSONDict, ODVInput

if TYPE_CHECKING:
    from telegram import Bot
    from telegram._payment.shippingoption import ShippingOption


class ShippingQuery(TelegramObject):
    """This object contains information about an incoming shipping query.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`id` is equal.

    Note:
        In Python :keyword:`from` is a reserved word. Use :paramref:`from_user` instead.

    Args:
        id (:obj:`str`): Unique query identifier.
        from_user (:class:`telegram.User`): User who sent the query.
        invoice_payload (:obj:`str`): Bot-specified invoice payload.
        shipping_address (:class:`telegram.ShippingAddress`): User specified shipping address.

    Attributes:
        id (:obj:`str`): Unique query identifier.
        from_user (:class:`telegram.User`): User who sent the query.
        invoice_payload (:obj:`str`): Bot-specified invoice payload.
        shipping_address (:class:`telegram.ShippingAddress`): User specified shipping address.


    """

    __slots__ = ("from_user", "id", "invoice_payload", "shipping_address")

    def __init__(
        self,
        id: str,  # pylint: disable=redefined-builtin
        from_user: User,
        invoice_payload: str,
        shipping_address: ShippingAddress,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.id: str = id
        self.from_user: User = from_user
        self.invoice_payload: str = invoice_payload
        self.shipping_address: ShippingAddress = shipping_address

        self._id_attrs = (self.id,)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "ShippingQuery":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["from_user"] = de_json_optional(data.pop("from", None), User, bot)
        data["shipping_address"] = de_json_optional(
            data.get("shipping_address"), ShippingAddress, bot
        )

        return super().de_json(data=data, bot=bot)

    async def answer(
        self,
        ok: bool,
        shipping_options: Sequence["ShippingOption"] | None = None,
        error_message: str | None = None,
        *,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
        api_kwargs: JSONDict | None = None,
    ) -> bool:
        """Shortcut for::

             await bot.answer_shipping_query(update.shipping_query.id, *args, **kwargs)

        For the documentation of the arguments, please see
        :meth:`telegram.Bot.answer_shipping_query`.

        """
        return await self.get_bot().answer_shipping_query(
            shipping_query_id=self.id,
            ok=ok,
            shipping_options=shipping_options,
            error_message=error_message,
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
            api_kwargs=api_kwargs,
        )



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import asyncio
import os
from pathlib import Path

import pytest

from telegram import Bot, ChatPhoto, Voice
from telegram.error import TelegramError
from telegram.request import RequestData
from tests.auxil.bot_method_checks import (
    check_defaults_handling,
    check_shortcut_call,
    check_shortcut_signature,
)
from tests.auxil.files import data_file
from tests.auxil.networking import expect_bad_request
from tests.auxil.slots import mro_slots


@pytest.fixture
def chatphoto_file():
    with data_file("telegram.jpg").open("rb") as f:
        yield f


@pytest.fixture(scope="module")
async def chat_photo(bot, super_group_id):
    async def func():
        return (await bot.get_chat(super_group_id, read_timeout=50)).photo

    return await expect_bad_request(
        func, "Type of file mismatch", "Telegram did not accept the file."
    )


class ChatPhotoTestBase:
    chatphoto_small_file_id = "smallCgADAQADngIAAuyVeEez0xRovKi9VAI"
    chatphoto_big_file_id = "bigCgADAQADngIAAuyVeEez0xRovKi9VAI"
    chatphoto_small_file_unique_id = "smalladc3145fd2e84d95b64d68eaa22aa33e"
    chatphoto_big_file_unique_id = "bigadc3145fd2e84d95b64d68eaa22aa33e"
    chatphoto_file_url = "https://python-telegram-bot.org/static/testfiles/telegram.jpg"


class TestChatPhotoWithoutRequest(ChatPhotoTestBase):
    def test_slot_behaviour(self, chat_photo):
        for attr in chat_photo.__slots__:
            assert getattr(chat_photo, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(chat_photo)) == len(set(mro_slots(chat_photo))), "duplicate slot"

    def test_de_json(self, offline_bot, chat_photo):
        json_dict = {
            "small_file_id": self.chatphoto_small_file_id,
            "big_file_id": self.chatphoto_big_file_id,
            "small_file_unique_id": self.chatphoto_small_file_unique_id,
            "big_file_unique_id": self.chatphoto_big_file_unique_id,
        }
        chat_photo = ChatPhoto.de_json(json_dict, offline_bot)
        assert chat_photo.api_kwargs == {}
        assert chat_photo.small_file_id == self.chatphoto_small_file_id
        assert chat_photo.big_file_id == self.chatphoto_big_file_id
        assert chat_photo.small_file_unique_id == self.chatphoto_small_file_unique_id
        assert chat_photo.big_file_unique_id == self.chatphoto_big_file_unique_id

    async def test_to_dict(self, chat_photo):
        chat_photo_dict = chat_photo.to_dict()

        assert isinstance(chat_photo_dict, dict)
        assert chat_photo_dict["small_file_id"] == chat_photo.small_file_id
        assert chat_photo_dict["big_file_id"] == chat_photo.big_file_id
        assert chat_photo_dict["small_file_unique_id"] == chat_photo.small_file_unique_id
        assert chat_photo_dict["big_file_unique_id"] == chat_photo.big_file_unique_id

    def test_equality(self):
        a = ChatPhoto(
            self.chatphoto_small_file_id,
            self.chatphoto_big_file_id,
            self.chatphoto_small_file_unique_id,
            self.chatphoto_big_file_unique_id,
        )
        b = ChatPhoto(
            self.chatphoto_small_file_id,
            self.chatphoto_big_file_id,
            self.chatphoto_small_file_unique_id,
            self.chatphoto_big_file_unique_id,
        )
        c = ChatPhoto(
            "", "", self.chatphoto_small_file_unique_id, self.chatphoto_big_file_unique_id
        )
        d = ChatPhoto("", "", 0, 0)
        e = Voice(self.chatphoto_small_file_id, self.chatphoto_small_file_unique_id, 0)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

    async def test_send_with_chat_photo(
        self, monkeypatch, offline_bot, super_group_id, chat_photo
    ):
        async def make_assertion(url, request_data: RequestData, *args, **kwargs):
            return request_data.parameters["photo"] == chat_photo.to_dict()

        monkeypatch.setattr(offline_bot.request, "post", make_assertion)
        message = await offline_bot.set_chat_photo(photo=chat_photo, chat_id=super_group_id)
        assert message

    async def test_get_small_file_instance_method(self, monkeypatch, chat_photo):
        async def make_assertion(*_, **kwargs):
            return kwargs["file_id"] == chat_photo.small_file_id

        assert check_shortcut_signature(ChatPhoto.get_small_file, Bot.get_file, ["file_id"], [])
        assert await check_shortcut_call(
            chat_photo.get_small_file, chat_photo.get_bot(), "get_file"
        )
        assert await check_defaults_handling(chat_photo.get_small_file, chat_photo.get_bot())

        monkeypatch.setattr(chat_photo.get_bot(), "get_file", make_assertion)
        assert await chat_photo.get_small_file()

    async def test_get_big_file_instance_method(self, monkeypatch, chat_photo):
        async def make_assertion(*_, **kwargs):
            return kwargs["file_id"] == chat_photo.big_file_id

        assert check_shortcut_signature(ChatPhoto.get_big_file, Bot.get_file, ["file_id"], [])
        assert await check_shortcut_call(chat_photo.get_big_file, chat_photo.get_bot(), "get_file")
        assert await check_defaults_handling(chat_photo.get_big_file, chat_photo.get_bot())

        monkeypatch.setattr(chat_photo.get_bot(), "get_file", make_assertion)
        assert await chat_photo.get_big_file()


class TestChatPhotoWithRequest:
    async def test_get_and_download(self, bot, chat_photo, tmp_file):
        tasks = {bot.get_file(chat_photo.small_file_id), bot.get_file(chat_photo.big_file_id)}
        asserts = []

        for task in asyncio.as_completed(tasks):
            file = await task
            if file.file_unique_id == chat_photo.small_file_unique_id:
                asserts.append("small")
            elif file.file_unique_id == chat_photo.big_file_unique_id:
                asserts.append("big")
            assert file.file_path.startswith("https://")

            await file.download_to_drive(tmp_file)
            assert tmp_file.is_file()

        assert "small" in asserts
        assert "big" in asserts

    async def test_send_all_args(self, bot, super_group_id, chatphoto_file):
        async def func():
            assert await bot.set_chat_photo(super_group_id, chatphoto_file)

        await expect_bad_request(
            func, "Type of file mismatch", "Telegram did not accept the file."
        )

    async def test_error_send_empty_file(self, bot, super_group_id):
        with Path(os.devnull).open("rb") as chatphoto_file, pytest.raises(TelegramError):
            await bot.set_chat_photo(chat_id=super_group_id, photo=chatphoto_file)

    async def test_error_send_empty_file_id(self, bot, super_group_id):
        with pytest.raises(TelegramError):
            await bot.set_chat_photo(chat_id=super_group_id, photo="")

    async def test_error_send_without_required_args(self, bot, super_group_id):
        with pytest.raises(TypeError):
            await bot.set_chat_photo(chat_id=super_group_id)



#!/usr/bin/env python
# flake8: noqa: E501
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
from copy import deepcopy

import pytest

from telegram import (
    Bot,
    Credentials,
    File,
    PassportData,
    PassportElementErrorDataField,
    PassportElementErrorSelfie,
    PassportFile,
)
from telegram.error import PassportDecryptionError

# Note: All classes in telegram.credentials (except EncryptedCredentials) aren't directly tested
# here, although they are implicitly tested. Testing for those classes was too much work and not
# worth it.
from telegram.request import RequestData
from tests.auxil.pytest_classes import make_bot
from tests.auxil.slots import mro_slots

RAW_PASSPORT_DATA = {
    "credentials": {
        "hash": "qB4hz2LMcXYhglwz6EvXMMyI3PURisWLXl/iCmCXcSk=",
        "secret": "O6x3X2JrLO1lUIhw48os1gaenDuZLhesoZMKXehZwtM3vsxOdtxHKWQyLNwtbyy4snYpARXDwf8f1QHNmQ/M1PwBQvk1ozrZBXb4a6k/iYj+P4v8Xw2M++CRHqZv0LaxFtyHOXnNYZ6dXpNeF0ZvYYTmm0FsYvK+/3/F6VDB3Oe6xWlXFLwaCCP/jA9i2+dKp6iq8NLOo4VnxenPKWWYz20RZ50MdAbS3UR+NCx4AHM2P5DEGrHNW0tMXJ+FG3jpVrit5BuCbB/eRgKxNGRWNxEGV5hun5MChdxKCEHCimnUA97h7MZdoTgYxkrfvDSZ/V89BnFXLdr87t/NLvVE0Q==",
        "data": "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",
    },
    "data": [
        {
            "data": "QRfzWcCN4WncvRO3lASG+d+c5gzqXtoCinQ1PgtYiZMKXCksx9eB9Ic1bOt8C/un9/XaX220PjJSO7Kuba+nXXC51qTsjqP9rnLKygnEIWjKrfiDdklzgcukpRzFSjiOAvhy86xFJZ1PfPSrFATy/Gp1RydLzbrBd2ZWxZqXrxcMoA0Q2UTTFXDoCYerEAiZoD69i79tB/6nkLBcUUvN5d52gKd/GowvxWqAAmdO6l1N7jlo6aWjdYQNBAK1KHbJdbRZMJLxC1MqMuZXAYrPoYBRKr5xAnxDTmPn/LEZKLc3gwwZyEgR5x7e9jp5heM6IEMmsv3O/6SUeEQs7P0iVuRSPLMJLfDdwns8Tl3fF2M4IxKVovjCaOVW+yHKsADDAYQPzzH2RcrWVD0TP5I64mzpK64BbTOq3qm3Hn51SV9uA/+LvdGbCp7VnzHx4EdUizHsVyilJULOBwvklsrDRvXMiWmh34ZSR6zilh051tMEcRf0I+Oe7pIxVJd/KKfYA2Z/eWVQTCn5gMuAInQNXFSqDIeIqBX+wca6kvOCUOXB7J2uRjTpLaC4DM9s/sNjSBvFixcGAngt+9oap6Y45rQc8ZJaNN/ALqEJAmkphW8=",
            "type": "personal_details",
            "hash": "What to put here?",
        },
        {
            "reverse_side": {
                "file_size": 32424112,
                "file_date": 1534074942,
                "file_id": "DgADBAADNQQAAtoagFPf4wwmFZdmyQI",
                "file_unique_id": "adc3145fd2e84d95b64d68eaa22aa33e",
            },
            "translation": [
                {
                    "file_size": 28640,
                    "file_date": 1535630933,
                    "file_id": "DgADBAADswMAAisqQVAmooP-kVgLgAI",
                    "file_unique_id": "52a90d53d6064bb58feb582acdc3a324",
                },
                {
                    "file_size": 28672,
                    "file_date": 1535630933,
                    "file_id": "DgADBAAD1QMAAnrpQFBMZsT3HysjwwI",
                    "file_unique_id": "7285f864d168441ba1f7d02146250432",
                },
            ],
            "front_side": {
                "file_size": 28624,
                "file_date": 1534074942,
                "file_id": "DgADBAADxwMAApnQgVPK2-ckL2eXVAI",
                "file_unique_id": "d9d52a700cbb4a189a80104aa5978133",
            },
            "type": "driver_license",
            "selfie": {
                "file_size": 28592,
                "file_date": 1534074942,
                "file_id": "DgADBAADEQQAAkopgFNr6oi-wISRtAI",
                "file_unique_id": "d4e390cca57b4da5a65322b304762a12",
            },
            "data": "eJUOFuY53QKmGqmBgVWlLBAQCUQJ79n405SX6M5aGFIIodOPQqnLYvMNqTwTrXGDlW+mVLZcbu+y8luLVO8WsJB/0SB7q5WaXn/IMt1G9lz5G/KMLIZG/x9zlnimsaQLg7u8srG6L4KZzv+xkbbHjZdETrxU8j0N/DoS4HvLMRSJAgeFUrY6v2YW9vSRg+fSxIqQy1jR2VKpzAT8OhOz7A==",
            "hash": "We seriously need to improve this mess! took so long to debug!",
        },
        {
            "translation": [
                {
                    "file_size": 28480,
                    "file_date": 1535630939,
                    "file_id": "DgADBAADyQUAAqyqQVC_eoX_KwNjJwI",
                    "file_unique_id": "38b2877b443542cbaf520c6e36a33ac4",
                },
                {
                    "file_size": 28528,
                    "file_date": 1535630939,
                    "file_id": "DgADBAADsQQAAubTQVDRO_FN3lOwWwI",
                    "file_unique_id": "f008ca48c44b4a47895ddbcd2f76741e",
                },
            ],
            "files": [
                {
                    "file_size": 28640,
                    "file_date": 1534074988,
                    "file_id": "DgADBAADLAMAAhwfgVMyfGa5Nr0LvAI",
                    "file_unique_id": "b170748794834644baaa3ec57ee4ce7a",
                },
                {
                    "file_size": 28480,
                    "file_date": 1534074988,
                    "file_id": "DgADBAADaQQAAsFxgVNVfLZuT-_3ZQI",
                    "file_unique_id": "19a12ae34dca424b85e0308f706cee75",
                },
            ],
            "type": "utility_bill",
            "hash": "Wow over 30 minutes spent debugging passport stuff.",
        },
        {
            "data": "j9SksVkSj128DBtZA+3aNjSFNirzv+R97guZaMgae4Gi0oDVNAF7twPR7j9VSmPedfJrEwL3O889Ei+a5F1xyLLyEI/qEBljvL70GFIhYGitS0JmNabHPHSZrjOl8b4s/0Z0Px2GpLO5siusTLQonimdUvu4UPjKquYISmlKEKhtmGATy+h+JDjNCYuOkhakeNw0Rk0BHgj0C3fCb7WZNQSyVb+2GTu6caR6eXf/AFwFp0TV3sRz3h0WIVPW8bna",
            "type": "address",
            "hash": "at least I get the pattern now",
        },
        {"email": "fb3e3i47zt@dispostable.com", "type": "email", "hash": "this should be it."},
    ],
}


@pytest.fixture(scope="module")
def all_passport_data():
    return [
        {
            "type": "personal_details",
            "data": RAW_PASSPORT_DATA["data"][0]["data"],
            "hash": "what to put here?",
        },
        {
            "type": "passport",
            "data": RAW_PASSPORT_DATA["data"][1]["data"],
            "front_side": RAW_PASSPORT_DATA["data"][1]["front_side"],
            "selfie": RAW_PASSPORT_DATA["data"][1]["selfie"],
            "translation": RAW_PASSPORT_DATA["data"][1]["translation"],
            "hash": "more data arghh",
        },
        {
            "type": "internal_passport",
            "data": RAW_PASSPORT_DATA["data"][1]["data"],
            "front_side": RAW_PASSPORT_DATA["data"][1]["front_side"],
            "selfie": RAW_PASSPORT_DATA["data"][1]["selfie"],
            "translation": RAW_PASSPORT_DATA["data"][1]["translation"],
            "hash": "more data arghh",
        },
        {
            "type": "driver_license",
            "data": RAW_PASSPORT_DATA["data"][1]["data"],
            "front_side": RAW_PASSPORT_DATA["data"][1]["front_side"],
            "reverse_side": RAW_PASSPORT_DATA["data"][1]["reverse_side"],
            "selfie": RAW_PASSPORT_DATA["data"][1]["selfie"],
            "translation": RAW_PASSPORT_DATA["data"][1]["translation"],
            "hash": "more data arghh",
        },
        {
            "type": "identity_card",
            "data": RAW_PASSPORT_DATA["data"][1]["data"],
            "front_side": RAW_PASSPORT_DATA["data"][1]["front_side"],
            "reverse_side": RAW_PASSPORT_DATA["data"][1]["reverse_side"],
            "selfie": RAW_PASSPORT_DATA["data"][1]["selfie"],
            "translation": RAW_PASSPORT_DATA["data"][1]["translation"],
            "hash": "more data arghh",
        },
        {
            "type": "utility_bill",
            "files": RAW_PASSPORT_DATA["data"][2]["files"],
            "translation": RAW_PASSPORT_DATA["data"][2]["translation"],
            "hash": "more data arghh",
        },
        {
            "type": "bank_statement",
            "files": RAW_PASSPORT_DATA["data"][2]["files"],
            "translation": RAW_PASSPORT_DATA["data"][2]["translation"],
            "hash": "more data arghh",
        },
        {
            "type": "rental_agreement",
            "files": RAW_PASSPORT_DATA["data"][2]["files"],
            "translation": RAW_PASSPORT_DATA["data"][2]["translation"],
            "hash": "more data arghh",
        },
        {
            "type": "passport_registration",
            "files": RAW_PASSPORT_DATA["data"][2]["files"],
            "translation": RAW_PASSPORT_DATA["data"][2]["translation"],
            "hash": "more data arghh",
        },
        {
            "type": "temporary_registration",
            "files": RAW_PASSPORT_DATA["data"][2]["files"],
            "translation": RAW_PASSPORT_DATA["data"][2]["translation"],
            "hash": "more data arghh",
        },
        {
            "type": "address",
            "data": RAW_PASSPORT_DATA["data"][3]["data"],
            "hash": "more data arghh",
        },
        {"type": "email", "email": "fb3e3i47zt@dispostable.com", "hash": "more data arghh"},
        {
            "type": "phone_number",
            "phone_number": "fb3e3i47zt@dispostable.com",
            "hash": "more data arghh",
        },
    ]


@pytest.fixture(scope="module")
def passport_data(bot):
    return PassportData.de_json(RAW_PASSPORT_DATA, bot=bot)


class PassportTestBase:
    driver_license_selfie_file_id = "DgADBAADEQQAAkopgFNr6oi-wISRtAI"
    driver_license_selfie_file_unique_id = "d4e390cca57b4da5a65322b304762a12"
    driver_license_front_side_file_id = "DgADBAADxwMAApnQgVPK2-ckL2eXVAI"
    driver_license_front_side_file_unique_id = "d9d52a700cbb4a189a80104aa5978133"
    driver_license_reverse_side_file_id = "DgADBAADNQQAAtoagFPf4wwmFZdmyQI"
    driver_license_reverse_side_file_unique_id = "adc3145fd2e84d95b64d68eaa22aa33e"
    driver_license_translation_1_file_id = "DgADBAADswMAAisqQVAmooP-kVgLgAI"
    driver_license_translation_1_file_unique_id = "52a90d53d6064bb58feb582acdc3a324"
    driver_license_translation_2_file_id = "DgADBAAD1QMAAnrpQFBMZsT3HysjwwI"
    driver_license_translation_2_file_unique_id = "7285f864d168441ba1f7d02146250432"
    utility_bill_1_file_id = "DgADBAADLAMAAhwfgVMyfGa5Nr0LvAI"
    utility_bill_1_file_unique_id = "b170748794834644baaa3ec57ee4ce7a"
    utility_bill_2_file_id = "DgADBAADaQQAAsFxgVNVfLZuT-_3ZQI"
    utility_bill_2_file_unique_id = "19a12ae34dca424b85e0308f706cee75"
    utility_bill_translation_1_file_id = "DgADBAADyQUAAqyqQVC_eoX_KwNjJwI"
    utility_bill_translation_1_file_unique_id = "38b2877b443542cbaf520c6e36a33ac4"
    utility_bill_translation_2_file_id = "DgADBAADsQQAAubTQVDRO_FN3lOwWwI"
    utility_bill_translation_2_file_unique_id = "f008ca48c44b4a47895ddbcd2f76741e"
    driver_license_selfie_credentials_file_hash = "Cila/qLXSBH7DpZFbb5bRZIRxeFW2uv/ulL0u0JNsYI="
    driver_license_selfie_credentials_secret = "tivdId6RNYNsvXYPppdzrbxOBuBOr9wXRPDcCvnXU7E="


class TestPassportWithoutRequest(PassportTestBase):
    def test_slot_behaviour(self, passport_data):
        inst = passport_data
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_creation(self, passport_data):
        assert isinstance(passport_data, PassportData)

    def test_expected_encrypted_values(self, passport_data):
        personal_details, driver_license, utility_bill, address, email = passport_data.data

        assert personal_details.type == "personal_details"
        assert personal_details.data == RAW_PASSPORT_DATA["data"][0]["data"]

        assert driver_license.type == "driver_license"
        assert driver_license.data == RAW_PASSPORT_DATA["data"][1]["data"]
        assert isinstance(driver_license.selfie, PassportFile)
        assert driver_license.selfie.file_id == self.driver_license_selfie_file_id
        assert driver_license.selfie.file_unique_id == self.driver_license_selfie_file_unique_id

        assert isinstance(driver_license.front_side, PassportFile)
        assert driver_license.front_side.file_id == self.driver_license_front_side_file_id
        assert (
            driver_license.front_side.file_unique_id
            == self.driver_license_front_side_file_unique_id
        )

        assert isinstance(driver_license.reverse_side, PassportFile)
        assert driver_license.reverse_side.file_id == self.driver_license_reverse_side_file_id
        assert (
            driver_license.reverse_side.file_unique_id
            == self.driver_license_reverse_side_file_unique_id
        )

        assert isinstance(driver_license.translation[0], PassportFile)
        assert driver_license.translation[0].file_id == self.driver_license_translation_1_file_id
        assert (
            driver_license.translation[0].file_unique_id
            == self.driver_license_translation_1_file_unique_id
        )

        assert isinstance(driver_license.translation[1], PassportFile)
        assert driver_license.translation[1].file_id == self.driver_license_translation_2_file_id
        assert (
            driver_license.translation[1].file_unique_id
            == self.driver_license_translation_2_file_unique_id
        )

        assert utility_bill.type == "utility_bill"
        assert isinstance(utility_bill.files[0], PassportFile)
        assert utility_bill.files[0].file_id == self.utility_bill_1_file_id
        assert utility_bill.files[0].file_unique_id == self.utility_bill_1_file_unique_id

        assert isinstance(utility_bill.files[1], PassportFile)
        assert utility_bill.files[1].file_id == self.utility_bill_2_file_id
        assert utility_bill.files[1].file_unique_id == self.utility_bill_2_file_unique_id

        assert isinstance(utility_bill.translation[0], PassportFile)
        assert utility_bill.translation[0].file_id == self.utility_bill_translation_1_file_id
        assert (
            utility_bill.translation[0].file_unique_id
            == self.utility_bill_translation_1_file_unique_id
        )

        assert isinstance(utility_bill.translation[1], PassportFile)
        assert utility_bill.translation[1].file_id == self.utility_bill_translation_2_file_id
        assert (
            utility_bill.translation[1].file_unique_id
            == self.utility_bill_translation_2_file_unique_id
        )

        assert address.type == "address"
        assert address.data == RAW_PASSPORT_DATA["data"][3]["data"]

        assert email.type == "email"
        assert email.email == "fb3e3i47zt@dispostable.com"

    def test_expected_decrypted_values(self, passport_data):
        (
            personal_details,
            driver_license,
            utility_bill,
            address,
            email,
        ) = passport_data.decrypted_data

        assert personal_details.type == "personal_details"
        assert personal_details.data.to_dict() == {
            "first_name": "FIRSTNAME",
            "middle_name": "MIDDLENAME",
            "first_name_native": "FIRSTNAMENATIVE",
            "residence_country_code": "DK",
            "birth_date": "01.01.2001",
            "last_name_native": "LASTNAMENATIVE",
            "gender": "female",
            "middle_name_native": "MIDDLENAMENATIVE",
            "country_code": "DK",
            "last_name": "LASTNAME",
        }

        assert driver_license.type == "driver_license"
        assert driver_license.data.to_dict() == {
            "expiry_date": "01.01.2001",
            "document_no": "DOCUMENT_NO",
        }
        assert isinstance(driver_license.selfie, PassportFile)
        assert driver_license.selfie.file_id == self.driver_license_selfie_file_id
        assert driver_license.selfie.file_unique_id == self.driver_license_selfie_file_unique_id

        assert isinstance(driver_license.front_side, PassportFile)
        assert driver_license.front_side.file_id == self.driver_license_front_side_file_id
        assert (
            driver_license.front_side.file_unique_id
            == self.driver_license_front_side_file_unique_id
        )

        assert isinstance(driver_license.reverse_side, PassportFile)
        assert driver_license.reverse_side.file_id == self.driver_license_reverse_side_file_id
        assert (
            driver_license.reverse_side.file_unique_id
            == self.driver_license_reverse_side_file_unique_id
        )

        assert address.type == "address"
        assert address.data.to_dict() == {
            "city": "CITY",
            "street_line2": "STREET_LINE2",
            "state": "STATE",
            "post_code": "POSTCODE",
            "country_code": "DK",
            "street_line1": "STREET_LINE1",
        }

        assert utility_bill.type == "utility_bill"
        assert isinstance(utility_bill.files[0], PassportFile)
        assert utility_bill.files[0].file_id == self.utility_bill_1_file_id
        assert utility_bill.files[0].file_unique_id == self.utility_bill_1_file_unique_id

        assert isinstance(utility_bill.files[1], PassportFile)
        assert utility_bill.files[1].file_id == self.utility_bill_2_file_id
        assert utility_bill.files[1].file_unique_id == self.utility_bill_2_file_unique_id

        assert email.type == "email"
        assert email.email == "fb3e3i47zt@dispostable.com"

    def test_de_json_and_to_dict(self, offline_bot):
        passport_data = PassportData.de_json(RAW_PASSPORT_DATA, offline_bot)
        assert passport_data.api_kwargs == {}
        assert passport_data.to_dict() == RAW_PASSPORT_DATA

        assert passport_data.decrypted_data
        assert passport_data.to_dict() == RAW_PASSPORT_DATA

    def test_equality(self, passport_data):
        a = PassportData(passport_data.data, passport_data.credentials)
        b = PassportData(passport_data.data, passport_data.credentials)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        new_pp_data = deepcopy(passport_data)
        new_pp_data.credentials._unfreeze()
        new_pp_data.credentials.hash = "NOTAPROPERHASH"
        c = PassportData(new_pp_data.data, new_pp_data.credentials)

        assert a != c
        assert hash(a) != hash(c)

    def test_bot_init_invalid_key(self, offline_bot):
        with pytest.raises(TypeError):
            Bot(offline_bot.token, private_key="Invalid key!")

        # Different error messages for different cryptography versions
        with pytest.raises(
            ValueError, match=r"(Could not deserialize key data)|(Unable to load PEM file)"
        ):
            Bot(offline_bot.token, private_key=b"Invalid key!")

    def test_all_types(self, passport_data, offline_bot, all_passport_data):
        credentials = passport_data.decrypted_credentials.to_dict()

        # Copy credentials from other types to all types so we can decrypt everything
        sd = credentials["secure_data"]
        credentials["secure_data"] = {
            "personal_details": sd["personal_details"].copy(),
            "passport": sd["driver_license"].copy(),
            "internal_passport": sd["driver_license"].copy(),
            "driver_license": sd["driver_license"].copy(),
            "identity_card": sd["driver_license"].copy(),
            "address": sd["address"].copy(),
            "utility_bill": sd["utility_bill"].copy(),
            "bank_statement": sd["utility_bill"].copy(),
            "rental_agreement": sd["utility_bill"].copy(),
            "passport_registration": sd["utility_bill"].copy(),
            "temporary_registration": sd["utility_bill"].copy(),
        }

        new = PassportData.de_json(
            {
                "data": all_passport_data,
                # Replaced below
                "credentials": {"data": "data", "hash": "hash", "secret": "secret"},
            },
            bot=offline_bot,
        )
        assert new.api_kwargs == {}

        new.credentials._decrypted_data = Credentials.de_json(credentials, offline_bot)
        assert new.credentials.api_kwargs == {}

        assert isinstance(new, PassportData)
        assert new.decrypted_data

    async def test_passport_data_okay_with_non_crypto_bot(self, offline_bot):
        async with make_bot(token=offline_bot.token) as b:
            assert PassportData.de_json(RAW_PASSPORT_DATA, bot=b)

    def test_wrong_hash(self, offline_bot):
        data = deepcopy(RAW_PASSPORT_DATA)
        data["credentials"]["hash"] = "bm90Y29ycmVjdGhhc2g="  # Not correct hash
        passport_data = PassportData.de_json(data, bot=offline_bot)
        with pytest.raises(PassportDecryptionError):
            assert passport_data.decrypted_data

    async def test_wrong_key(self, offline_bot):
        short_key = (
            b"-----BEGIN RSA PRIVATE"
            b" KEY-----\r\nMIIBOQIBAAJBAKU+OZ2jJm7sCA/ec4gngNZhXYPu+DZ/TAwSMl0W7vAPXAsLplBk\r\nO8l6IBHx8N0ZC4Bc65mO3b2G8YAzqndyqH8CAwEAAQJAWOx3jQFzeVXDsOaBPdAk\r\nYTncXVeIc6tlfUl9mOLyinSbRNCy1XicOiOZFgH1rRKOGIC1235QmqxFvdecySoY\r\nwQIhAOFeGgeX9CrEPuSsd9+kqUcA2avCwqdQgSdy2qggRFyJAiEAu7QHT8JQSkHU\r\nDELfzrzc24AhjyG0z1DpGZArM8COascCIDK42SboXj3Z2UXiQ0CEcMzYNiVgOisq\r\nBUd5pBi+2mPxAiAM5Z7G/Sv1HjbKrOGh29o0/sXPhtpckEuj5QMC6E0gywIgFY6S\r\nNjwrAA+cMmsgY0O2fAzEKkDc5YiFsiXaGaSS4eA=\r\n-----END"
            b" RSA PRIVATE KEY-----"
        )
        async with make_bot(token=offline_bot.token, private_key=short_key) as b:
            passport_data = PassportData.de_json(RAW_PASSPORT_DATA, bot=b)
            with pytest.raises(PassportDecryptionError):
                assert passport_data.decrypted_data

        async with make_bot(token=offline_bot.token, private_key=short_key) as b:
            passport_data = PassportData.de_json(RAW_PASSPORT_DATA, bot=b)
            with pytest.raises(PassportDecryptionError):
                assert passport_data.decrypted_data

    async def test_mocked_download_passport_file(self, passport_data, monkeypatch):
        # The files are not coming from our test offline_bot, therefore the file id is invalid/wrong
        # when coming from this offline_bot, so we monkeypatch the call, to make sure that Bot.get_file
        # at least gets called
        # TODO: Actually download a passport file in a test
        selfie = passport_data.decrypted_data[1].selfie

        # NOTE: file_unique_id is not used in the get_file method, so it is passed directly
        async def get_file(*_, **kwargs):
            return File(kwargs["file_id"], selfie.file_unique_id)

        monkeypatch.setattr(passport_data.get_bot(), "get_file", get_file)
        file = await selfie.get_file()
        assert file.file_id == selfie.file_id
        assert file.file_unique_id == selfie.file_unique_id
        assert file._credentials.file_hash == self.driver_license_selfie_credentials_file_hash
        assert file._credentials.secret == self.driver_license_selfie_credentials_secret

    async def test_mocked_set_passport_data_errors(
        self, monkeypatch, offline_bot, chat_id, passport_data
    ):
        async def make_assertion(url, request_data: RequestData, *args, **kwargs):
            data = request_data.parameters
            return (
                data["user_id"] == str(chat_id)
                and data["errors"][0]["file_hash"]
                == (
                    passport_data.decrypted_credentials.secure_data.driver_license.selfie.file_hash
                )
                and data["errors"][1]["data_hash"]
                == passport_data.decrypted_credentials.secure_data.driver_license.data.data_hash
            )

        monkeypatch.setattr(offline_bot.request, "post", make_assertion)
        message = await offline_bot.set_passport_data_errors(
            chat_id,
            [
                PassportElementErrorSelfie(
                    "driver_license",
                    (
                        passport_data.decrypted_credentials.secure_data.driver_license.selfie.file_hash
                    ),
                    "You're not handsome enough to use this app!",
                ),
                PassportElementErrorDataField(
                    "driver_license",
                    "expiry_date",
                    (
                        passport_data.decrypted_credentials.secure_data.driver_license.data.data_hash
                    ),
                    "Your driver license is expired!",
                ),
            ],
        )
        assert message



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import pytest

from telegram import ReplyKeyboardRemove
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def reply_keyboard_remove():
    return ReplyKeyboardRemove(selective=ReplyKeyboardRemoveTestBase.selective)


class ReplyKeyboardRemoveTestBase:
    remove_keyboard = True
    selective = True


class TestReplyKeyboardRemoveWithoutRequest(ReplyKeyboardRemoveTestBase):
    def test_slot_behaviour(self, reply_keyboard_remove):
        inst = reply_keyboard_remove
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_expected_values(self, reply_keyboard_remove):
        assert reply_keyboard_remove.remove_keyboard == self.remove_keyboard
        assert reply_keyboard_remove.selective == self.selective

    def test_to_dict(self, reply_keyboard_remove):
        reply_keyboard_remove_dict = reply_keyboard_remove.to_dict()

        assert (
            reply_keyboard_remove_dict["remove_keyboard"] == reply_keyboard_remove.remove_keyboard
        )
        assert reply_keyboard_remove_dict["selective"] == reply_keyboard_remove.selective


class TestReplyKeyboardRemoveWithRequest(ReplyKeyboardRemoveTestBase):
    async def test_send_message_with_reply_keyboard_remove(
        self, bot, chat_id, reply_keyboard_remove
    ):
        message = await bot.send_message(chat_id, "Text", reply_markup=reply_keyboard_remove)
        assert message.text == "Text"



#!/usr/bin/env python
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import pytest

from telegram import Dice, InputChecklist, InputChecklistTask, MessageEntity
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def input_checklist_task():
    return InputChecklistTask(
        id=InputChecklistTaskTestBase.id,
        text=InputChecklistTaskTestBase.text,
        parse_mode=InputChecklistTaskTestBase.parse_mode,
        text_entities=InputChecklistTaskTestBase.text_entities,
    )


class InputChecklistTaskTestBase:
    id = 1
    text = "buy food"
    parse_mode = "MarkdownV2"
    text_entities = [
        MessageEntity(type="bold", offset=0, length=3),
        MessageEntity(type="italic", offset=4, length=4),
    ]


class TestInputChecklistTaskWithoutRequest(InputChecklistTaskTestBase):
    def test_slot_behaviour(self, input_checklist_task):
        for attr in input_checklist_task.__slots__:
            assert getattr(input_checklist_task, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(input_checklist_task)) == len(set(mro_slots(input_checklist_task))), (
            "duplicate slot"
        )

    def test_expected_values(self, input_checklist_task):
        assert input_checklist_task.id == self.id
        assert input_checklist_task.text == self.text
        assert input_checklist_task.parse_mode == self.parse_mode
        assert input_checklist_task.text_entities == tuple(self.text_entities)

    def test_to_dict(self, input_checklist_task):
        iclt_dict = input_checklist_task.to_dict()

        assert isinstance(iclt_dict, dict)
        assert iclt_dict["id"] == self.id
        assert iclt_dict["text"] == self.text
        assert iclt_dict["parse_mode"] == self.parse_mode
        assert iclt_dict["text_entities"] == [entity.to_dict() for entity in self.text_entities]

        # Test that default-value parameter `parse_mode` is handled correctly
        input_checklist_task = InputChecklistTask(id=1, text="text")
        iclt_dict = input_checklist_task.to_dict()
        assert "parse_mode" not in iclt_dict

    def test_equality(self, input_checklist_task):
        a = input_checklist_task
        b = InputChecklistTask(id=self.id, text=f"other {self.text}")
        c = InputChecklistTask(id=self.id + 1, text=self.text)
        d = Dice(value=1, emoji="🎲")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)


@pytest.fixture(scope="module")
def input_checklist():
    return InputChecklist(
        title=InputChecklistTestBase.title,
        tasks=InputChecklistTestBase.tasks,
        parse_mode=InputChecklistTestBase.parse_mode,
        title_entities=InputChecklistTestBase.title_entities,
        others_can_add_tasks=InputChecklistTestBase.others_can_add_tasks,
        others_can_mark_tasks_as_done=InputChecklistTestBase.others_can_mark_tasks_as_done,
    )


class InputChecklistTestBase:
    title = "test list"
    tasks = [
        InputChecklistTask(id=1, text="eat"),
        InputChecklistTask(id=2, text="sleep"),
    ]
    parse_mode = "MarkdownV2"
    title_entities = [
        MessageEntity(type="bold", offset=0, length=4),
        MessageEntity(type="italic", offset=5, length=4),
    ]
    others_can_add_tasks = True
    others_can_mark_tasks_as_done = False


class TestInputChecklistWithoutRequest(InputChecklistTestBase):
    def test_slot_behaviour(self, input_checklist):
        for attr in input_checklist.__slots__:
            assert getattr(input_checklist, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(input_checklist)) == len(set(mro_slots(input_checklist))), (
            "duplicate slot"
        )

    def test_expected_values(self, input_checklist):
        assert input_checklist.title == self.title
        assert input_checklist.tasks == tuple(self.tasks)
        assert input_checklist.parse_mode == self.parse_mode
        assert input_checklist.title_entities == tuple(self.title_entities)
        assert input_checklist.others_can_add_tasks == self.others_can_add_tasks
        assert input_checklist.others_can_mark_tasks_as_done == self.others_can_mark_tasks_as_done

    def test_to_dict(self, input_checklist):
        icl_dict = input_checklist.to_dict()

        assert isinstance(icl_dict, dict)
        assert icl_dict["title"] == self.title
        assert icl_dict["tasks"] == [task.to_dict() for task in self.tasks]
        assert icl_dict["parse_mode"] == self.parse_mode
        assert icl_dict["title_entities"] == [entity.to_dict() for entity in self.title_entities]
        assert icl_dict["others_can_add_tasks"] == self.others_can_add_tasks
        assert icl_dict["others_can_mark_tasks_as_done"] == self.others_can_mark_tasks_as_done

        # Test that default-value parameter `parse_mode` is handled correctly
        input_checklist = InputChecklist(title=self.title, tasks=self.tasks)
        icl_dict = input_checklist.to_dict()
        assert "parse_mode" not in icl_dict

    def test_equality(self, input_checklist):
        a = input_checklist
        b = InputChecklist(
            title=f"other {self.title}",
            tasks=[InputChecklistTask(id=1, text="eat"), InputChecklistTask(id=2, text="sleep")],
        )
        c = InputChecklist(
            title=self.title,
            tasks=[InputChecklistTask(id=9, text="Other Task")],
        )
        d = Dice(value=1, emoji="🎲")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].

import datetime as dtm

import pytest

from telegram import Dice
from telegram._chat import Chat
from telegram._message import Message
from telegram._payment.stars.staramount import StarAmount
from telegram._suggestedpost import (
    SuggestedPostApprovalFailed,
    SuggestedPostApproved,
    SuggestedPostDeclined,
    SuggestedPostInfo,
    SuggestedPostPaid,
    SuggestedPostParameters,
    SuggestedPostPrice,
    SuggestedPostRefunded,
)
from telegram._utils.datetime import UTC, to_timestamp
from telegram.constants import SuggestedPostInfoState
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def suggested_post_parameters():
    return SuggestedPostParameters(
        price=SuggestedPostParametersTestBase.price,
        send_date=SuggestedPostParametersTestBase.send_date,
    )


class SuggestedPostParametersTestBase:
    price = SuggestedPostPrice(currency="XTR", amount=100)
    send_date = dtm.datetime.now(tz=UTC).replace(microsecond=0)


class TestSuggestedPostParametersWithoutRequest(SuggestedPostParametersTestBase):
    def test_slot_behaviour(self, suggested_post_parameters):
        for attr in suggested_post_parameters.__slots__:
            assert getattr(suggested_post_parameters, attr, "err") != "err", (
                f"got extra slot '{attr}'"
            )
        assert len(mro_slots(suggested_post_parameters)) == len(
            set(mro_slots(suggested_post_parameters))
        ), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "price": self.price.to_dict(),
            "send_date": to_timestamp(self.send_date),
        }
        spp = SuggestedPostParameters.de_json(json_dict, offline_bot)
        assert spp.price == self.price
        assert spp.send_date == self.send_date
        assert spp.api_kwargs == {}

    def test_de_json_localization(self, offline_bot, raw_bot, tz_bot):
        json_dict = {
            "price": self.price.to_dict(),
            "send_date": to_timestamp(self.send_date),
        }

        spp_bot = SuggestedPostParameters.de_json(json_dict, offline_bot)
        spp_bot_raw = SuggestedPostParameters.de_json(json_dict, raw_bot)
        spp_bot_tz = SuggestedPostParameters.de_json(json_dict, tz_bot)

        # comparing utcoffsets because comparing tzinfo objects is not reliable
        send_date_offset = spp_bot_tz.send_date.utcoffset()
        send_date_offset_tz = tz_bot.defaults.tzinfo.utcoffset(
            spp_bot_tz.send_date.replace(tzinfo=None)
        )

        assert spp_bot.send_date.tzinfo == UTC
        assert spp_bot_raw.send_date.tzinfo == UTC
        assert send_date_offset_tz == send_date_offset

    def test_to_dict(self, suggested_post_parameters):
        spp_dict = suggested_post_parameters.to_dict()

        assert isinstance(spp_dict, dict)
        assert spp_dict["price"] == self.price.to_dict()
        assert spp_dict["send_date"] == to_timestamp(self.send_date)

    def test_equality(self, suggested_post_parameters):
        a = suggested_post_parameters
        b = SuggestedPostParameters(price=self.price, send_date=self.send_date)
        c = SuggestedPostParameters(
            price=self.price, send_date=self.send_date + dtm.timedelta(seconds=1)
        )
        e = Dice(4, "emoji")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != e
        assert hash(a) != hash(e)


@pytest.fixture(scope="module")
def suggested_post_info():
    return SuggestedPostInfo(
        state=SuggestedPostInfoTestBase.state,
        price=SuggestedPostInfoTestBase.price,
        send_date=SuggestedPostInfoTestBase.send_date,
    )


class SuggestedPostInfoTestBase:
    state = SuggestedPostInfoState.PENDING
    price = SuggestedPostPrice(currency="XTR", amount=100)
    send_date = dtm.datetime.now(tz=UTC).replace(microsecond=0)


class TestSuggestedPostInfoWithoutRequest(SuggestedPostInfoTestBase):
    def test_slot_behaviour(self, suggested_post_info):
        for attr in suggested_post_info.__slots__:
            assert getattr(suggested_post_info, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(suggested_post_info)) == len(set(mro_slots(suggested_post_info))), (
            "duplicate slot"
        )

    def test_type_enum_conversion(self):
        assert type(SuggestedPostInfo("pending").state) is SuggestedPostInfoState
        assert SuggestedPostInfo("unknown").state == "unknown"

    def test_de_json(self, offline_bot):
        json_dict = {
            "state": self.state,
            "price": self.price.to_dict(),
            "send_date": to_timestamp(self.send_date),
        }
        spi = SuggestedPostInfo.de_json(json_dict, offline_bot)
        assert spi.state == self.state
        assert spi.price == self.price
        assert spi.send_date == self.send_date
        assert spi.api_kwargs == {}

    def test_de_json_localization(self, offline_bot, raw_bot, tz_bot):
        json_dict = {
            "state": self.state,
            "price": self.price.to_dict(),
            "send_date": to_timestamp(self.send_date),
        }

        spi_bot = SuggestedPostInfo.de_json(json_dict, offline_bot)
        spi_bot_raw = SuggestedPostInfo.de_json(json_dict, raw_bot)
        spi_bot_tz = SuggestedPostInfo.de_json(json_dict, tz_bot)

        # comparing utcoffsets because comparing tzinfo objects is not reliable
        send_date_offset = spi_bot_tz.send_date.utcoffset()
        send_date_offset_tz = tz_bot.defaults.tzinfo.utcoffset(
            spi_bot_tz.send_date.replace(tzinfo=None)
        )

        assert spi_bot.send_date.tzinfo == UTC
        assert spi_bot_raw.send_date.tzinfo == UTC
        assert send_date_offset_tz == send_date_offset

    def test_to_dict(self, suggested_post_info):
        spi_dict = suggested_post_info.to_dict()

        assert isinstance(spi_dict, dict)
        assert spi_dict["state"] == self.state
        assert spi_dict["price"] == self.price.to_dict()
        assert spi_dict["send_date"] == to_timestamp(self.send_date)

    def test_equality(self, suggested_post_info):
        a = suggested_post_info
        b = SuggestedPostInfo(state=self.state, price=self.price)
        c = SuggestedPostInfo(state=SuggestedPostInfoState.DECLINED, price=self.price)
        e = Dice(4, "emoji")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != e
        assert hash(a) != hash(e)


@pytest.fixture(scope="module")
def suggested_post_price():
    return SuggestedPostPrice(
        currency=SuggestedPostPriceTestBase.currency,
        amount=SuggestedPostPriceTestBase.amount,
    )


class SuggestedPostPriceTestBase:
    currency = "XTR"
    amount = 100


class TestSuggestedPostPriceWithoutRequest(SuggestedPostPriceTestBase):
    def test_slot_behaviour(self, suggested_post_price):
        for attr in suggested_post_price.__slots__:
            assert getattr(suggested_post_price, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(suggested_post_price)) == len(set(mro_slots(suggested_post_price))), (
            "duplicate slot"
        )

    def test_de_json(self, offline_bot):
        json_dict = {
            "currency": self.currency,
            "amount": self.amount,
        }
        spp = SuggestedPostPrice.de_json(json_dict, offline_bot)
        assert spp.currency == self.currency
        assert spp.amount == self.amount
        assert spp.api_kwargs == {}

    def test_to_dict(self, suggested_post_price):
        spp_dict = suggested_post_price.to_dict()

        assert isinstance(spp_dict, dict)
        assert spp_dict["currency"] == self.currency
        assert spp_dict["amount"] == self.amount

    def test_equality(self, suggested_post_price):
        a = suggested_post_price
        b = SuggestedPostPrice(currency=self.currency, amount=self.amount)
        c = SuggestedPostPrice(currency="TON", amount=self.amount)
        e = Dice(4, "emoji")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != e
        assert hash(a) != hash(e)


@pytest.fixture(scope="module")
def suggested_post_declined():
    return SuggestedPostDeclined(
        suggested_post_message=SuggestedPostDeclinedTestBase.suggested_post_message,
        comment=SuggestedPostDeclinedTestBase.comment,
    )


class SuggestedPostDeclinedTestBase:
    suggested_post_message = Message(1, dtm.datetime.now(), Chat(1, ""), text="post this pls.")
    comment = "another time"


class TestSuggestedPostDeclinedWithoutRequest(SuggestedPostDeclinedTestBase):
    def test_slot_behaviour(self, suggested_post_declined):
        for attr in suggested_post_declined.__slots__:
            assert getattr(suggested_post_declined, attr, "err") != "err", (
                f"got extra slot '{attr}'"
            )
        assert len(mro_slots(suggested_post_declined)) == len(
            set(mro_slots(suggested_post_declined))
        ), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "suggested_post_message": self.suggested_post_message.to_dict(),
            "comment": self.comment,
        }
        spd = SuggestedPostDeclined.de_json(json_dict, offline_bot)
        assert spd.suggested_post_message == self.suggested_post_message
        assert spd.comment == self.comment
        assert spd.api_kwargs == {}

    def test_to_dict(self, suggested_post_declined):
        spd_dict = suggested_post_declined.to_dict()

        assert isinstance(spd_dict, dict)
        assert spd_dict["suggested_post_message"] == self.suggested_post_message.to_dict()
        assert spd_dict["comment"] == self.comment

    def test_equality(self, suggested_post_declined):
        a = suggested_post_declined
        b = SuggestedPostDeclined(
            suggested_post_message=self.suggested_post_message, comment=self.comment
        )
        c = SuggestedPostDeclined(suggested_post_message=self.suggested_post_message, comment="no")
        e = Dice(4, "emoji")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != e
        assert hash(a) != hash(e)


@pytest.fixture(scope="module")
def suggested_post_paid():
    return SuggestedPostPaid(
        currency=SuggestedPostPaidTestBase.currency,
        suggested_post_message=SuggestedPostPaidTestBase.suggested_post_message,
        amount=SuggestedPostPaidTestBase.amount,
        star_amount=SuggestedPostPaidTestBase.star_amount,
    )


class SuggestedPostPaidTestBase:
    suggested_post_message = Message(1, dtm.datetime.now(), Chat(1, ""), text="post this pls.")
    currency = "XTR"
    amount = 100
    star_amount = StarAmount(100)


class TestSuggestedPostPaidWithoutRequest(SuggestedPostPaidTestBase):
    def test_slot_behaviour(self, suggested_post_paid):
        for attr in suggested_post_paid.__slots__:
            assert getattr(suggested_post_paid, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(suggested_post_paid)) == len(set(mro_slots(suggested_post_paid))), (
            "duplicate slot"
        )

    def test_de_json(self, offline_bot):
        json_dict = {
            "suggested_post_message": self.suggested_post_message.to_dict(),
            "currency": self.currency,
            "amount": self.amount,
            "star_amount": self.star_amount.to_dict(),
        }
        spp = SuggestedPostPaid.de_json(json_dict, offline_bot)
        assert spp.suggested_post_message == self.suggested_post_message
        assert spp.currency == self.currency
        assert spp.amount == self.amount
        assert spp.star_amount == self.star_amount
        assert spp.api_kwargs == {}

    def test_to_dict(self, suggested_post_paid):
        spp_dict = suggested_post_paid.to_dict()

        assert isinstance(spp_dict, dict)
        assert spp_dict["suggested_post_message"] == self.suggested_post_message.to_dict()
        assert spp_dict["currency"] == self.currency
        assert spp_dict["amount"] == self.amount
        assert spp_dict["star_amount"] == self.star_amount.to_dict()

    def test_equality(self, suggested_post_paid):
        a = suggested_post_paid
        b = SuggestedPostPaid(
            suggested_post_message=self.suggested_post_message,
            currency=self.currency,
            amount=self.amount,
            star_amount=self.star_amount,
        )
        c = SuggestedPostPaid(
            suggested_post_message=self.suggested_post_message,
            currency=self.currency,
            amount=self.amount - 1,
            star_amount=StarAmount(self.amount - 1),
        )
        e = Dice(4, "emoji")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != e
        assert hash(a) != hash(e)


@pytest.fixture(scope="module")
def suggested_post_refunded():
    return SuggestedPostRefunded(
        reason=SuggestedPostRefundedTestBase.reason,
        suggested_post_message=SuggestedPostRefundedTestBase.suggested_post_message,
    )


class SuggestedPostRefundedTestBase:
    reason = "post_deleted"
    suggested_post_message = Message(1, dtm.datetime.now(), Chat(1, ""), text="post this pls.")


class TestSuggestedPostRefundedWithoutRequest(SuggestedPostRefundedTestBase):
    def test_slot_behaviour(self, suggested_post_refunded):
        for attr in suggested_post_refunded.__slots__:
            assert getattr(suggested_post_refunded, attr, "err") != "err", (
                f"got extra slot '{attr}'"
            )
        assert len(mro_slots(suggested_post_refunded)) == len(
            set(mro_slots(suggested_post_refunded))
        ), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "suggested_post_message": self.suggested_post_message.to_dict(),
            "reason": self.reason,
        }
        spr = SuggestedPostRefunded.de_json(json_dict, offline_bot)
        assert spr.suggested_post_message == self.suggested_post_message
        assert spr.reason == self.reason
        assert spr.api_kwargs == {}

    def test_to_dict(self, suggested_post_refunded):
        spr_dict = suggested_post_refunded.to_dict()

        assert isinstance(spr_dict, dict)
        assert spr_dict["suggested_post_message"] == self.suggested_post_message.to_dict()
        assert spr_dict["reason"] == self.reason

    def test_equality(self, suggested_post_refunded):
        a = suggested_post_refunded
        b = SuggestedPostRefunded(
            suggested_post_message=self.suggested_post_message, reason=self.reason
        )
        c = SuggestedPostRefunded(
            suggested_post_message=self.suggested_post_message, reason="payment_refunded"
        )
        e = Dice(4, "emoji")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != e
        assert hash(a) != hash(e)


@pytest.fixture(scope="module")
def suggested_post_approved():
    return SuggestedPostApproved(
        send_date=SuggestedPostApprovedTestBase.send_date,
        suggested_post_message=SuggestedPostApprovedTestBase.suggested_post_message,
        price=SuggestedPostApprovedTestBase.price,
    )


class SuggestedPostApprovedTestBase:
    send_date = dtm.datetime.now(tz=UTC).replace(microsecond=0)
    suggested_post_message = Message(1, dtm.datetime.now(), Chat(1, ""), text="post this pls.")
    price = SuggestedPostPrice(currency="XTR", amount=100)


class TestSuggestedPostApprovedWithoutRequest(SuggestedPostApprovedTestBase):
    def test_slot_behaviour(self, suggested_post_approved):
        for attr in suggested_post_approved.__slots__:
            assert getattr(suggested_post_approved, attr, "err") != "err", (
                f"got extra slot '{attr}'"
            )
        assert len(mro_slots(suggested_post_approved)) == len(
            set(mro_slots(suggested_post_approved))
        ), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "send_date": to_timestamp(self.send_date),
            "suggested_post_message": self.suggested_post_message.to_dict(),
            "price": self.price.to_dict(),
        }
        spa = SuggestedPostApproved.de_json(json_dict, offline_bot)
        assert spa.send_date == self.send_date
        assert spa.suggested_post_message == self.suggested_post_message
        assert spa.price == self.price
        assert spa.api_kwargs == {}

    def test_de_json_localization(self, offline_bot, raw_bot, tz_bot):
        json_dict = {
            "send_date": to_timestamp(self.send_date),
            "suggested_post_message": self.suggested_post_message.to_dict(),
            "price": self.price.to_dict(),
        }

        spa_bot = SuggestedPostApproved.de_json(json_dict, offline_bot)
        spa_bot_raw = SuggestedPostApproved.de_json(json_dict, raw_bot)
        spi_bot_tz = SuggestedPostApproved.de_json(json_dict, tz_bot)

        # comparing utcoffsets because comparing tzinfo objects is not reliable
        send_date_offset = spi_bot_tz.send_date.utcoffset()
        send_date_offset_tz = tz_bot.defaults.tzinfo.utcoffset(
            spi_bot_tz.send_date.replace(tzinfo=None)
        )

        assert spa_bot.send_date.tzinfo == UTC
        assert spa_bot_raw.send_date.tzinfo == UTC
        assert send_date_offset_tz == send_date_offset

    def test_to_dict(self, suggested_post_approved):
        spa_dict = suggested_post_approved.to_dict()

        assert isinstance(spa_dict, dict)
        assert spa_dict["send_date"] == to_timestamp(self.send_date)
        assert spa_dict["suggested_post_message"] == self.suggested_post_message.to_dict()
        assert spa_dict["price"] == self.price.to_dict()

    def test_equality(self, suggested_post_approved):
        a = suggested_post_approved
        b = SuggestedPostApproved(
            send_date=self.send_date,
            suggested_post_message=self.suggested_post_message,
            price=self.price,
        )
        c = SuggestedPostApproved(
            send_date=self.send_date,
            suggested_post_message=self.suggested_post_message,
            price=SuggestedPostPrice(currency="XTR", amount=self.price.amount - 1),
        )
        e = Dice(4, "emoji")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != e
        assert hash(a) != hash(e)


@pytest.fixture(scope="module")
def suggested_post_approval_failed():
    return SuggestedPostApprovalFailed(
        price=SuggestedPostApprovalFailedTestBase.price,
        suggested_post_message=SuggestedPostApprovalFailedTestBase.suggested_post_message,
    )


class SuggestedPostApprovalFailedTestBase:
    price = SuggestedPostPrice(currency="XTR", amount=100)
    suggested_post_message = Message(1, dtm.datetime.now(), Chat(1, ""), text="post this pls.")


class TestSuggestedPostApprovalFailedWithoutRequest(SuggestedPostApprovalFailedTestBase):
    def test_slot_behaviour(self, suggested_post_approval_failed):
        for attr in suggested_post_approval_failed.__slots__:
            assert getattr(suggested_post_approval_failed, attr, "err") != "err", (
                f"got extra slot '{attr}'"
            )
        assert len(mro_slots(suggested_post_approval_failed)) == len(
            set(mro_slots(suggested_post_approval_failed))
        ), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "price": self.price.to_dict(),
            "suggested_post_message": self.suggested_post_message.to_dict(),
        }
        spaf = SuggestedPostApprovalFailed.de_json(json_dict, offline_bot)
        assert spaf.price == self.price
        assert spaf.suggested_post_message == self.suggested_post_message
        assert spaf.api_kwargs == {}

    def test_to_dict(self, suggested_post_approval_failed):
        spaf_dict = suggested_post_approval_failed.to_dict()

        assert isinstance(spaf_dict, dict)
        assert spaf_dict["price"] == self.price.to_dict()
        assert spaf_dict["suggested_post_message"] == self.suggested_post_message.to_dict()

    def test_equality(self, suggested_post_approval_failed):
        a = suggested_post_approval_failed
        b = SuggestedPostApprovalFailed(
            price=self.price,
            suggested_post_message=self.suggested_post_message,
        )
        c = SuggestedPostApprovalFailed(
            price=SuggestedPostPrice(currency="XTR", amount=self.price.amount - 1),
            suggested_post_message=self.suggested_post_message,
        )
        e = Dice(4, "emoji")

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != e
        assert hash(a) != hash(e)



#
#  A library that provides a Python interface to the Telegram Bot API
#  Copyright (C) 2015-2026
#  Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser Public License for more details.
#
#  You should have received a copy of the GNU Lesser Public License
#  along with this program.  If not, see [http://www.gnu.org/licenses/].
import inspect

keyword_args = [
    "Keyword Arguments:",
    (
        "    read_timeout (:obj:`float` | :obj:`None`, optional): Value to pass to "
        "        :paramref:`telegram.request.BaseRequest.post.read_timeout`. Defaults to "
        "        :attr:`~telegram.request.BaseRequest.DEFAULT_NONE`. "
    ),
    (
        "    write_timeout (:obj:`float` | :obj:`None`, optional): Value to pass to "
        "        :paramref:`telegram.request.BaseRequest.post.write_timeout`. Defaults to "
        "        :attr:`~telegram.request.BaseRequest.DEFAULT_NONE`."
    ),
    (
        "    connect_timeout (:obj:`float` | :obj:`None`, optional): Value to pass to "
        "        :paramref:`telegram.request.BaseRequest.post.connect_timeout`. Defaults to "
        "        :attr:`~telegram.request.BaseRequest.DEFAULT_NONE`."
    ),
    (
        "    pool_timeout (:obj:`float` | :obj:`None`, optional): Value to pass to "
        "        :paramref:`telegram.request.BaseRequest.post.pool_timeout`. Defaults to "
        "        :attr:`~telegram.request.BaseRequest.DEFAULT_NONE`."
    ),
    (
        "    api_kwargs (:obj:`dict`, optional): Arbitrary keyword arguments"
        "        to be passed to the Telegram API. See :meth:`~telegram.Bot.do_api_request` for"
        "        limitations."
    ),
    "",
]

media_write_timeout_change_methods = [
    "add_sticker_to_set",
    "create_new_sticker_set",
    "send_animation",
    "send_audio",
    "send_document",
    "send_media_group",
    "send_photo",
    "send_sticker",
    "send_video",
    "send_video_note",
    "send_voice",
    "set_chat_photo",
    "upload_sticker_file",
]
media_write_timeout_change = [
    "    write_timeout (:obj:`float` | :obj:`None`, optional): Value to pass to "
    "        :paramref:`telegram.request.BaseRequest.post.write_timeout`. By default, ``20`` "
    "        seconds are used as write timeout."
    "",
    "",
    "       .. versionchanged:: 22.0",
    "           The default value changed to "
    "           :attr:`~telegram.request.BaseRequest.DEFAULT_NONE`.",
    "",
    "",
]
get_updates_read_timeout_addition = [
    "        :paramref:`timeout` will be added to this value.",
    "",
    "",
    "        .. versionchanged:: 20.7",
    "           Defaults to :attr:`~telegram.request.BaseRequest.DEFAULT_NONE` instead of ",
    "           ``2``.",
]


def find_insert_pos_for_kwargs(lines: list[str]) -> int:
    """Finds the correct position to insert the keyword arguments and returns the index."""
    for idx, value in reversed(list(enumerate(lines))):  # reversed since :returns: is at the end
        if value.startswith("Returns"):
            return idx
    return False


def check_timeout_and_api_kwargs_presence(obj: object) -> int:
    """Checks if the method has timeout and api_kwargs keyword only parameters."""
    sig = inspect.signature(obj)
    params_to_check = (
        "read_timeout",
        "write_timeout",
        "connect_timeout",
        "pool_timeout",
        "api_kwargs",
    )
    return all(
        param in sig.parameters and sig.parameters[param].kind == inspect.Parameter.KEYWORD_ONLY
        for param in params_to_check
    )



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import datetime as dtm
from collections.abc import Sequence

import pytest

from telegram import (
    InputFile,
    InputMediaPhoto,
    InputMediaVideo,
    InputProfilePhotoAnimated,
    InputProfilePhotoStatic,
    InputSticker,
    InputStoryContentPhoto,
    InputStoryContentVideo,
    MessageEntity,
)
from telegram.constants import ChatType
from telegram.request._requestparameter import RequestParameter
from tests.auxil.files import data_file
from tests.auxil.slots import mro_slots


class TestRequestParameterWithoutRequest:
    def test_slot_behaviour(self):
        inst = RequestParameter("name", "value", [1, 2])
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_init(self):
        request_parameter = RequestParameter("name", "value", [1, 2])
        assert request_parameter.name == "name"
        assert request_parameter.value == "value"
        assert request_parameter.input_files == [1, 2]

        request_parameter = RequestParameter("name", "value", None)
        assert request_parameter.name == "name"
        assert request_parameter.value == "value"
        assert request_parameter.input_files is None

    @pytest.mark.parametrize(
        ("value", "expected"),
        [
            (1, "1"),
            ("one", "one"),
            (True, "true"),
            (None, None),
            ([1, "1"], '[1, "1"]'),
            ({True: None}, '{"true": null}'),
            ((1,), "[1]"),
        ],
    )
    def test_json_value(self, value, expected):
        request_parameter = RequestParameter("name", value, None)
        assert request_parameter.json_value == expected

    def test_multiple_multipart_data(self):
        assert RequestParameter("name", "value", []).multipart_data is None

        input_file_1 = InputFile("data1", attach=True)
        input_file_2 = InputFile("data2", filename="custom")
        request_parameter = RequestParameter(
            value="value", name="name", input_files=[input_file_1, input_file_2]
        )
        files = request_parameter.multipart_data
        assert files[input_file_1.attach_name] == input_file_1.field_tuple
        assert files["name"] == input_file_2.field_tuple

    @pytest.mark.parametrize(
        ("value", "expected_value"),
        [
            (True, True),
            ("str", "str"),
            ({1: 1.0}, {1: 1.0}),
            (ChatType.PRIVATE, "private"),
            (MessageEntity("type", 1, 1), {"type": "type", "offset": 1, "length": 1}),
            (dtm.datetime(2019, 11, 11, 0, 26, 16, 10**5), 1573431976),
            (dtm.timedelta(days=42), 42 * 24 * 60 * 60),
            (
                [
                    True,
                    "str",
                    MessageEntity("type", 1, 1),
                    ChatType.PRIVATE,
                    dtm.datetime(2019, 11, 11, 0, 26, 16, 10**5),
                ],
                [True, "str", {"type": "type", "offset": 1, "length": 1}, "private", 1573431976],
            ),
        ],
    )
    def test_from_input_no_media(self, value, expected_value):
        request_parameter = RequestParameter.from_input("key", value)
        assert request_parameter.value == expected_value
        assert request_parameter.input_files is None

    @pytest.mark.parametrize(
        ("value", "expected_type", "expected_value"),
        [
            (dtm.timedelta(seconds=1), int, 1),
            (dtm.timedelta(milliseconds=1), float, 0.001),
        ],
    )
    def test_from_input_timedelta(self, value, expected_type, expected_value):
        request_parameter = RequestParameter.from_input("key", value)
        assert request_parameter.value == expected_value
        assert request_parameter.input_files is None
        assert isinstance(request_parameter.value, expected_type)

    def test_from_input_inputfile(self):
        inputfile_1 = InputFile("data1", filename="inputfile_1", attach=True)
        inputfile_2 = InputFile("data2", filename="inputfile_2")

        request_parameter = RequestParameter.from_input("key", inputfile_1)
        assert request_parameter.value == inputfile_1.attach_uri
        assert request_parameter.input_files == [inputfile_1]

        request_parameter = RequestParameter.from_input("key", inputfile_2)
        assert request_parameter.value is None
        assert request_parameter.input_files == [inputfile_2]

        request_parameter = RequestParameter.from_input("key", [inputfile_1, inputfile_2])
        assert request_parameter.value == [inputfile_1.attach_uri]
        assert request_parameter.input_files == [inputfile_1, inputfile_2]

    def test_from_input_input_media(self):
        input_media_no_thumb = InputMediaPhoto(media=data_file("telegram.jpg").read_bytes())
        input_media_thumb = InputMediaVideo(
            media=data_file("telegram.mp4").read_bytes(),
            thumbnail=data_file("telegram.jpg").read_bytes(),
        )

        request_parameter = RequestParameter.from_input("key", input_media_no_thumb)
        expected_no_thumb = input_media_no_thumb.to_dict()
        expected_no_thumb.update({"media": input_media_no_thumb.media.attach_uri})
        assert request_parameter.value == expected_no_thumb
        assert request_parameter.input_files == [input_media_no_thumb.media]

        request_parameter = RequestParameter.from_input("key", input_media_thumb)
        expected_thumb = input_media_thumb.to_dict()
        expected_thumb.update({"media": input_media_thumb.media.attach_uri})
        expected_thumb.update({"thumbnail": input_media_thumb.thumbnail.attach_uri})
        assert request_parameter.value == expected_thumb
        assert request_parameter.input_files == [
            input_media_thumb.media,
            input_media_thumb.thumbnail,
        ]

        request_parameter = RequestParameter.from_input(
            "key", [input_media_thumb, input_media_no_thumb]
        )
        assert request_parameter.value == [expected_thumb, expected_no_thumb]
        assert request_parameter.input_files == [
            input_media_thumb.media,
            input_media_thumb.thumbnail,
            input_media_no_thumb.media,
        ]

    def test_from_input_inputmedia_without_attach(self):
        """This case will never happen, but we test it for completeness"""
        input_media = InputMediaVideo(
            data_file("telegram.png").read_bytes(),
            thumbnail=data_file("telegram.png").read_bytes(),
            parse_mode=None,
        )
        input_media.media.attach_name = None
        input_media.thumbnail.attach_name = None
        request_parameter = RequestParameter.from_input("key", input_media)
        assert request_parameter.value == {"type": "video"}
        assert request_parameter.input_files == [input_media.media, input_media.thumbnail]

    def test_from_input_profile_photo_static(self):
        input_profile_photo = InputProfilePhotoStatic(data_file("telegram.jpg").read_bytes())
        expected = input_profile_photo.to_dict()
        expected.update({"photo": input_profile_photo.photo.attach_uri})
        request_parameter = RequestParameter.from_input("key", input_profile_photo)
        assert request_parameter.value == expected
        assert request_parameter.input_files == [input_profile_photo.photo]

    def test_from_input_profile_photo_animated(self):
        input_profile_photo = InputProfilePhotoAnimated(
            data_file("telegram2.mp4").read_bytes(),
            main_frame_timestamp=dtm.timedelta(seconds=42, milliseconds=43),
        )
        expected = input_profile_photo.to_dict()
        expected.update({"animation": input_profile_photo.animation.attach_uri})
        request_parameter = RequestParameter.from_input("key", input_profile_photo)
        assert request_parameter.value == expected
        assert request_parameter.input_files == [input_profile_photo.animation]

    @pytest.mark.parametrize(
        ("cls", "args"),
        [
            (InputProfilePhotoStatic, (data_file("telegram.jpg"),)),
            (
                InputProfilePhotoAnimated,
                (data_file("telegram2.mp4"), dtm.timedelta(seconds=42, milliseconds=43)),
            ),
        ],
    )
    def test_from_input_profile_photo_local_files(self, cls, args):
        input_profile_photo = cls(*args)
        expected = input_profile_photo.to_dict()
        requested = RequestParameter.from_input("key", input_profile_photo)
        assert requested.value == expected
        assert requested.input_files is None

    def test_from_input_inputsticker(self):
        input_sticker = InputSticker(data_file("telegram.png").read_bytes(), ["emoji"], "static")
        expected = input_sticker.to_dict()
        expected.update({"sticker": input_sticker.sticker.attach_uri})
        request_parameter = RequestParameter.from_input("key", input_sticker)
        assert request_parameter.value == expected
        assert request_parameter.input_files == [input_sticker.sticker]

    def test_from_input_story_content_photo(self):
        input_story_content_photo = InputStoryContentPhoto(data_file("telegram.jpg").read_bytes())
        expected = input_story_content_photo.to_dict()
        expected.update({"photo": input_story_content_photo.photo.attach_uri})
        request_parameter = RequestParameter.from_input("key", input_story_content_photo)
        assert request_parameter.value == expected
        assert request_parameter.input_files == [input_story_content_photo.photo]

    def test_from_input_story_content_video(self):
        input_story_content_video = InputStoryContentVideo(data_file("telegram2.mp4").read_bytes())
        expected = input_story_content_video.to_dict()
        expected.update({"video": input_story_content_video.video.attach_uri})
        request_parameter = RequestParameter.from_input("key", input_story_content_video)
        assert request_parameter.value == expected
        assert request_parameter.input_files == [input_story_content_video.video]

    @pytest.mark.parametrize(
        ("cls", "arg"),
        [
            (InputStoryContentPhoto, data_file("telegram.jpg")),
            (InputStoryContentVideo, data_file("telegram2.mp4")),
        ],
    )
    def test_from_input_story_content_local_files(self, cls, arg):
        input_story_content = cls(arg)
        expected = input_story_content.to_dict()
        requested = RequestParameter.from_input("key", input_story_content)
        assert requested.value == expected
        assert requested.input_files is None

    def test_from_input_str_and_bytes(self):
        input_str = "test_input"
        request_parameter = RequestParameter.from_input("input", input_str)
        assert request_parameter.value == input_str
        assert request_parameter.name == "input"
        assert request_parameter.input_files is None

        input_bytes = b"test_input"
        request_parameter = RequestParameter.from_input("input", input_bytes)
        assert request_parameter.value == input_bytes
        assert request_parameter.name == "input"
        assert request_parameter.input_files is None

    def test_from_input_different_sequences(self):
        input_list = ["item1", "item2"]
        request_parameter = RequestParameter.from_input("input", input_list)
        assert request_parameter.value == input_list
        assert request_parameter.name == "input"
        assert request_parameter.input_files is None

        input_tuple = tuple(input_list)
        request_parameter = RequestParameter.from_input("input", input_tuple)
        assert request_parameter.value == input_list
        assert request_parameter.name == "input"
        assert request_parameter.input_files is None

        class CustomSequence(Sequence):
            def __init__(self, items):
                self.items = items

            def __getitem__(self, index):
                return self.items[index]

            def __len__(self):
                return len(self.items)

        input_custom_sequence = CustomSequence(input_list)
        request_parameter = RequestParameter.from_input("input", input_custom_sequence)
        assert request_parameter.value == input_list
        assert request_parameter.name == "input"
        assert request_parameter.input_files is None



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains two objects used for request chats/users service messages."""

from collections.abc import Sequence
from typing import TYPE_CHECKING

from telegram._files.photosize import PhotoSize
from telegram._telegramobject import TelegramObject
from telegram._utils.argumentparsing import de_list_optional, parse_sequence_arg
from telegram._utils.types import JSONDict
from telegram._utils.usernames import get_full_name, get_link, get_name

if TYPE_CHECKING:
    from telegram._bot import Bot


class UsersShared(TelegramObject):
    """
    This object contains information about the user whose identifier was shared with the bot
    using a :class:`telegram.KeyboardButtonRequestUsers` button.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`request_id` and :attr:`users` are equal.

    .. versionadded:: 20.8
       Bot API 7.0 replaces ``UserShared`` with this class. The only difference is that now
       the ``user_ids`` is a sequence instead of a single integer.

    .. versionchanged:: 21.1
       The argument :attr:`users` is now considered for the equality comparison instead of
       ``user_ids``.

    .. versionremoved:: 21.2
       Removed the deprecated argument and attribute ``user_ids``.

    Args:
        request_id (:obj:`int`): Identifier of the request.
        users (Sequence[:class:`telegram.SharedUser`]): Information about users shared with the
            bot.

            .. versionadded:: 21.1

            .. versionchanged:: 21.2
               This argument is now required.

    Attributes:
        request_id (:obj:`int`): Identifier of the request.
        users (tuple[:class:`telegram.SharedUser`]): Information about users shared with the
            bot.

            .. versionadded:: 21.1
    """

    __slots__ = ("request_id", "users")

    def __init__(
        self,
        request_id: int,
        users: Sequence["SharedUser"],
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.request_id: int = request_id
        self.users: tuple[SharedUser, ...] = parse_sequence_arg(users)

        self._id_attrs = (self.request_id, self.users)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "UsersShared":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["users"] = de_list_optional(data.get("users"), SharedUser, bot)

        api_kwargs = {}
        # This is a deprecated field that TG still returns for backwards compatibility
        # Let's filter it out to speed up the de-json process
        if user_ids := data.get("user_ids"):
            api_kwargs = {"user_ids": user_ids}

        return super()._de_json(data=data, bot=bot, api_kwargs=api_kwargs)


class ChatShared(TelegramObject):
    """
    This object contains information about the chat whose identifier was shared with the bot
    using a :class:`telegram.KeyboardButtonRequestChat` button.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`request_id` and :attr:`chat_id` are equal.

    .. versionadded:: 20.1

    Args:
        request_id (:obj:`int`): Identifier of the request.
        chat_id (:obj:`int`): Identifier of the shared user. This number may be greater than 32
            bits and some programming languages may have difficulty/silent defects in interpreting
            it. But it is smaller than 52 bits, so a signed 64-bit integer or double-precision
            float type are safe for storing this identifier.
        title (:obj:`str`, optional): Title of the chat, if the title was requested by the bot.

            .. versionadded:: 21.1
        username (:obj:`str`, optional): Username of the chat, if the username was requested by
            the bot and available.

            .. versionadded:: 21.1
        photo (Sequence[:class:`telegram.PhotoSize`], optional): Available sizes of the chat photo,
            if the photo was requested by the bot

            .. versionadded:: 21.1

    Attributes:
        request_id (:obj:`int`): Identifier of the request.
        chat_id (:obj:`int`): Identifier of the shared user. This number may be greater than 32
            bits and some programming languages may have difficulty/silent defects in interpreting
            it. But it is smaller than 52 bits, so a signed 64-bit integer or double-precision
            float type are safe for storing this identifier.
        title (:obj:`str`): Optional. Title of the chat, if the title was requested by the bot.

            .. versionadded:: 21.1
        username (:obj:`str`): Optional. Username of the chat, if the username was requested by
            the bot and available.

            .. versionadded:: 21.1
        photo (tuple[:class:`telegram.PhotoSize`]): Optional. Available sizes of the chat photo,
            if the photo was requested by the bot

            .. versionadded:: 21.1
    """

    __slots__ = ("chat_id", "photo", "request_id", "title", "username")

    def __init__(
        self,
        request_id: int,
        chat_id: int,
        title: str | None = None,
        username: str | None = None,
        photo: Sequence[PhotoSize] | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.request_id: int = request_id
        self.chat_id: int = chat_id
        self.title: str | None = title
        self.username: str | None = username
        self.photo: tuple[PhotoSize, ...] | None = parse_sequence_arg(photo)

        self._id_attrs = (self.request_id, self.chat_id)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "ChatShared":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["photo"] = de_list_optional(data.get("photo"), PhotoSize, bot)
        return super().de_json(data=data, bot=bot)

    @property
    def link(self) -> str | None:
        """:obj:`str`: Convenience property. If :attr:`username` is available, returns a t.me link
        of the chat.

        .. versionadded:: 22.4
        """
        return get_link(self)


class SharedUser(TelegramObject):
    """
    This object contains information about a user that was shared with the bot using a
    :class:`telegram.KeyboardButtonRequestUsers` button.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`user_id` is equal.

    .. versionadded:: 21.1

    Args:
        user_id (:obj:`int`): Identifier of the shared user. This number may have 32 significant
            bits and some programming languages may have difficulty/silent defects in interpreting
            it. But it has atmost 52 significant bits, so 64-bit integers or double-precision
            float types are safe for storing these identifiers. The bot may not have access to the
            user and could be unable to use this identifier, unless the user is already known to
            the bot by some other means.
        first_name (:obj:`str`, optional): First name of the user, if the name was requested by the
            bot.
        last_name (:obj:`str`, optional): Last name of the user, if the name was requested by the
            bot.
        username (:obj:`str`, optional): Username of the user, if the username was requested by the
            bot.
        photo (Sequence[:class:`telegram.PhotoSize`], optional): Available sizes of the chat photo,
            if the photo was requested by the bot.

    Attributes:
        user_id (:obj:`int`): Identifier of the shared user. This number may have 32 significant
            bits and some programming languages may have difficulty/silent defects in interpreting
            it. But it has atmost 52 significant bits, so 64-bit integers or double-precision
            float types are safe for storing these identifiers. The bot may not have access to the
            user and could be unable to use this identifier, unless the user is already known to
            the bot by some other means.
        first_name (:obj:`str`): Optional. First name of the user, if the name was requested by the
            bot.
        last_name (:obj:`str`): Optional. Last name of the user, if the name was requested by the
            bot.
        username (:obj:`str`): Optional. Username of the user, if the username was requested by the
            bot.
        photo (tuple[:class:`telegram.PhotoSize`]): Available sizes of the chat photo, if
            the photo was requested by the bot. This list is empty if the photo was not requsted.
    """

    __slots__ = ("first_name", "last_name", "photo", "user_id", "username")

    def __init__(
        self,
        user_id: int,
        first_name: str | None = None,
        last_name: str | None = None,
        username: str | None = None,
        photo: Sequence[PhotoSize] | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        self.user_id: int = user_id
        self.first_name: str | None = first_name
        self.last_name: str | None = last_name
        self.username: str | None = username
        self.photo: tuple[PhotoSize, ...] | None = parse_sequence_arg(photo)

        self._id_attrs = (self.user_id,)

        self._freeze()

    @property
    def name(self) -> str | None:
        """:obj:`str`: Convenience property. If available, returns the user's :attr:`username`
        prefixed with "@". If :attr:`username` is not available, returns :attr:`full_name`.

        .. versionadded:: 22.4
        """
        return get_name(self)

    @property
    def full_name(self) -> str | None:
        """:obj:`str`: Convenience property. If :attr:`first_name` is not :obj:`None`, gives
        :attr:`first_name` followed by (if available) :attr:`last_name`.

        .. versionadded:: 22.4
        """
        return get_full_name(self)

    @property
    def link(self) -> str | None:
        """:obj:`str`: Convenience property. If :attr:`username` is available, returns a t.me link
        of the user.

        .. versionadded:: 22.4
        """
        return get_link(self)

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "SharedUser":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["photo"] = de_list_optional(data.get("photo"), PhotoSize, bot)
        return super().de_json(data=data, bot=bot)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains the StringCommandHandler class."""

from typing import TYPE_CHECKING, Any

from telegram._utils.defaultvalue import DEFAULT_TRUE
from telegram._utils.types import DVType
from telegram.ext._handlers.basehandler import BaseHandler
from telegram.ext._utils.types import CCT, RT, HandlerCallback

if TYPE_CHECKING:
    from telegram.ext import Application


class StringCommandHandler(BaseHandler[str, CCT, RT]):
    """Handler class to handle string commands. Commands are string updates that start with
    ``/``. The handler will add a :obj:`list` to the
    :class:`CallbackContext` named :attr:`CallbackContext.args`. It will contain a list of strings,
    which is the text following the command split on single whitespace characters.

    Note:
        This handler is not used to handle Telegram :class:`telegram.Update`, but strings manually
        put in the queue. For example to send messages with the bot using command line or API.

    Warning:
        When setting :paramref:`block` to :obj:`False`, you cannot rely on adding custom
        attributes to :class:`telegram.ext.CallbackContext`. See its docs for more info.

    Args:
        command (:obj:`str`): The command this handler should listen for.
        callback (:term:`coroutine function`): The callback function for this handler. Will be
            called when :meth:`check_update` has determined that an update should be processed by
            this handler. Callback signature::

                async def callback(update: str, context: CallbackContext)

            The return value of the callback is usually ignored except for the special case of
            :class:`telegram.ext.ConversationHandler`.
        block (:obj:`bool`, optional): Determines whether the return value of the callback should
            be awaited before processing the next handler in
            :meth:`telegram.ext.Application.process_update`. Defaults to :obj:`True`.

            .. seealso:: :wiki:`Concurrency`

    Attributes:
        command (:obj:`str`): The command this handler should listen for.
        callback (:term:`coroutine function`): The callback function for this handler.
        block (:obj:`bool`): Determines whether the return value of the callback should be
            awaited before processing the next handler in
            :meth:`telegram.ext.Application.process_update`.

    """

    __slots__ = ("command",)

    def __init__(
        self: "StringCommandHandler[CCT, RT]",
        command: str,
        callback: HandlerCallback[str, CCT, RT],
        block: DVType[bool] = DEFAULT_TRUE,
    ):
        super().__init__(callback, block=block)
        self.command: str = command

    def check_update(self, update: object) -> list[str] | None:
        """Determines whether an update should be passed to this handler's :attr:`callback`.

        Args:
            update (:obj:`object`): The incoming update.

        Returns:
            list[:obj:`str`]: List containing the text command split on whitespace.

        """
        if isinstance(update, str) and update.startswith("/"):
            args = update[1:].split(" ")
            if args[0] == self.command:
                return args[1:]
        return None

    def collect_additional_context(
        self,
        context: CCT,
        update: str,  # noqa: ARG002
        application: "Application[Any, CCT, Any, Any, Any, Any]",  # noqa: ARG002
        check_result: list[str] | None,
    ) -> None:
        """Add text after the command to :attr:`CallbackContext.args` as list, split on single
        whitespaces.
        """
        context.args = check_result



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains the base class for handlers as used by the Application."""

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Any, Generic, TypeVar

from telegram._utils.defaultvalue import DEFAULT_TRUE
from telegram._utils.repr import build_repr_with_selected_attrs
from telegram._utils.types import DVType
from telegram.ext._utils.types import CCT, HandlerCallback

if TYPE_CHECKING:
    from telegram.ext import Application

RT = TypeVar("RT")
UT = TypeVar("UT")


class BaseHandler(ABC, Generic[UT, CCT, RT]):
    """The base class for all update handlers. Create custom handlers by inheriting from it.

    Warning:
        When setting :paramref:`block` to :obj:`False`, you cannot rely on adding custom
        attributes to :class:`telegram.ext.CallbackContext`. See its docs for more info.

    This class is a :class:`~typing.Generic` class and accepts three type variables:

    1. The type of the updates that this handler will handle. Must coincide with the type of the
       first argument of :paramref:`callback`. :meth:`check_update` must only accept
       updates of this type.
    2. The type of the second argument of :paramref:`callback`. Must coincide with the type of the
       parameters :paramref:`handle_update.context` and
       :paramref:`collect_additional_context.context` as well as the second argument of
       :paramref:`callback`. Must be either :class:`~telegram.ext.CallbackContext` or a subclass
       of that class.

       .. tip::
           For this type variable, one should usually provide a :class:`~typing.TypeVar` that is
           also used for the mentioned method arguments. That way, a type checker can check whether
           this handler fits the definition of the :class:`~Application`.
    3. The return type of the :paramref:`callback` function accepted by this handler.

    .. seealso:: :wiki:`Types of Handlers <Types-of-Handlers>`

    .. versionchanged:: 20.0

        * The attribute ``run_async`` is now :paramref:`block`.
        * This class was previously named ``Handler``.

    Args:
        callback (:term:`coroutine function`): The callback function for this handler. Will be
            called when :meth:`check_update` has determined that an update should be processed by
            this handler. Callback signature::

                async def callback(update: Update, context: CallbackContext)

            The return value of the callback is usually ignored except for the special case of
            :class:`telegram.ext.ConversationHandler`.
        block (:obj:`bool`, optional): Determines whether the return value of the callback should
            be awaited before processing the next handler in
            :meth:`telegram.ext.Application.process_update`. Defaults to :obj:`True`.

            .. seealso:: :wiki:`Concurrency`

    Attributes:
        callback (:term:`coroutine function`): The callback function for this handler.
        block (:obj:`bool`): Determines whether the callback will run in a blocking way.

    """

    __slots__ = (
        "block",
        "callback",
    )

    def __init__(
        self: "BaseHandler[UT, CCT, RT]",
        callback: HandlerCallback[UT, CCT, RT],
        block: DVType[bool] = DEFAULT_TRUE,
    ):
        self.callback: HandlerCallback[UT, CCT, RT] = callback
        self.block: DVType[bool] = block

    def __repr__(self) -> str:
        """Give a string representation of the handler in the form ``ClassName[callback=...]``.

        As this class doesn't implement :meth:`object.__str__`, the default implementation
        will be used, which is equivalent to :meth:`__repr__`.

        Returns:
            :obj:`str`
        """
        try:
            callback_name = self.callback.__qualname__
        except AttributeError:
            callback_name = repr(self.callback)
        return build_repr_with_selected_attrs(self, callback=callback_name)

    @abstractmethod
    def check_update(self, update: object) -> bool | object | None:
        """
        This method is called to determine if an update should be handled by
        this handler instance. It should always be overridden.

        Note:
            Custom updates types can be handled by the application. Therefore, an implementation of
            this method should always check the type of :paramref:`update`.

        Args:
            update (:obj:`object` | :class:`telegram.Update`): The update to be tested.

        Returns:
            Either :obj:`None` or :obj:`False` if the update should not be handled. Otherwise an
            object that will be passed to :meth:`handle_update` and
            :meth:`collect_additional_context` when the update gets handled.

        """

    async def handle_update(
        self,
        update: UT,
        application: "Application[Any, CCT, Any, Any, Any, Any]",
        check_result: object,
        context: CCT,
    ) -> RT:
        """
        This method is called if it was determined that an update should indeed
        be handled by this instance. Calls :attr:`callback` along with its respectful
        arguments. To work with the :class:`telegram.ext.ConversationHandler`, this method
        returns the value returned from :attr:`callback`.
        Note that it can be overridden if needed by the subclassing handler.

        Args:
            update (:obj:`str` | :class:`telegram.Update`): The update to be handled.
            application (:class:`telegram.ext.Application`): The calling application.
            check_result (:class:`object`): The result from :meth:`check_update`.
            context (:class:`telegram.ext.CallbackContext`): The context as provided by
                the application.

        """
        self.collect_additional_context(context, update, application, check_result)
        return await self.callback(update, context)

    def collect_additional_context(
        self,
        context: CCT,
        update: UT,
        application: "Application[Any, CCT, Any, Any, Any, Any]",
        check_result: Any,
    ) -> None:
        """Prepares additional arguments for the context. Override if needed.

        Args:
            context (:class:`telegram.ext.CallbackContext`): The context object.
            update (:class:`telegram.Update`): The update to gather chat/user id from.
            application (:class:`telegram.ext.Application`): The calling application.
            check_result: The result (return value) from :meth:`check_update`.

        """
